from lib.common.os import mkdir, get_file_name, copy_file

"""
mpirun --allow-run-as-root -np 3 generax --families /data/families_plants.txt --species-tree /data/SpeciesTree_rooted.txt --rec-model UndatedDL --per-family-rates  --prefix test --max-spr-radius 3
"""


def prepare_OG_work_dir(OGs, sp_info_dict, top_work_dir, orthofinder_style=False):
    mkdir(top_work_dir, True)

    sp_list = list(sp_info_dict.keys())

    used_og_list = []
    for og_id in OGs.OG_dict:
        og = OGs.OG_dict[og_id]

        # filter not good OG, species number more than 3, single species not more than 50 gene, top two species not have more than 90% gene
        # species more than 3
        if len([i for i in og.species_stat if og.species_stat[i] > 0]) < 3:
            continue
        if max(og.species_stat.values()) > 50:
            continue
        if sum(sorted(og.species_stat.values())[-2:]) / sum(og.species_stat.values()) > 0.9:
            continue
        if sum(og.species_stat.values()) <= 3:
            continue

        # copy file
        og_dir = top_work_dir + "/" + og_id
        mkdir(og_dir, True)

        copy_file(og.tree_file, og_dir)
        copy_file(og.msa_file, og_dir)

        og.tree_file = og_dir + "/" + get_file_name(og.tree_file)
        og.msa_file = og_dir + "/" + get_file_name(og.msa_file)

        # map file
        map_file = og_dir + "/" + og_id + ".map"
        with open(map_file, 'w') as f:
            for gene in og.gene_list:
                if orthofinder_style:
                    f.write("%s_%s\t%s\n" %
                            (gene.species, gene.id, gene.species))
                else:
                    f.write("%s\t%s\n" % (gene.id, gene.species))

        og.map_file = map_file

        used_og_list.append(og_id)

    with open(top_work_dir+"/families.txt", 'w') as f:
        f.write("[FAMILIES]\n")
        for og_id in used_og_list:
            og = OGs.OG_dict[og_id]
            print_string = """- %s
starting_gene_tree = %s
alignment = %s
mapping = %s
subst_model = LG+G            
""" % (og_id, og.tree_file, og.msa_file, og.map_file)
            f.write(print_string)


if __name__ == '__main__':

    OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder/pt_file/OrthoFinder/Results_Jun01/Orthogroups/Orthogroups.tsv"
    species_tree_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder/pt_file/OrthoFinder/Results_Jun01/Species_Tree/SpeciesTree_rooted.txt"
    ref_xlsx = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/Gel_ref.xlsx'
    gene_tree_dir = "/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder/pt_file/OrthoFinder/Results_Jun01/Gene_Trees"
    msa_dir = "/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder/pt_file/OrthoFinder/Results_Jun01/MultipleSequenceAlignments"
    top_work_dir = "/lustre/home/xuyuxing/Work/Gel/orcWGD/generax/top_data"

    from lib.common.evolution.orthotools2 import OrthoGroups, read_species_info
    from Bio import Phylo

    # species tree
    species_tree = Phylo.read(species_tree_file, 'newick')
    sp_list = [i.name for i in species_tree.get_terminals()]

    # orthogroups
    OGs = OrthoGroups(OG_tsv_file=OG_tsv_file, species_list=sp_list)

    # read a standard species info file: sp_id, taxon_id, species_name, genome_file, gff_file, pt_file, cDNA_file, cds_file
    sp_info_dict = read_species_info(ref_xlsx)
    sp_info_dict = {i: sp_info_dict[i] for i in sp_info_dict if i in sp_list}

    # get tree and msa
    for i in OGs.OG_dict:
        OGs.OG_dict[i].tree_file = "%s/%s_tree.txt" % (gene_tree_dir, i)
        OGs.OG_dict[i].msa_file = "%s/%s.fa" % (msa_dir, i)

    # prepare_OG_work_dir
    prepare_OG_work_dir(OGs, sp_info_dict, top_work_dir, True)



from api.xuyuxing.genome.repeat_anno import get_aa_seq
from lib.common.os import cmd_run, get_file_dir, mkdir, multiprocess_running, rmdir
from lib.common.genome.seq_base import read_fasta_by_faidx
from src.xuyuxing.tools.seqtools import split_fasta
import os
import re
from lib.xuyuxing.base.base_function import merge_file

def ProteinByPfam_main(args):
    """
    class abc():
        pass

    args = abc()
    args.genome_seq_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Lindenbergia_luchunensis/repeat_by_EDTA/test/test/LTR.Copia.fasta.cd_hit'
    args.pfam_db_file = '/lustre/home/xuyuxing/Database/Pfam/20200516/Pfam-A.hmm'
    args.output_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Lindenbergia_luchunensis/repeat_by_EDTA/test/test/LTR.Copia.fasta.cd_hit.2'
    args.tmp_work_dir = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Lindenbergia_luchunensis/repeat_by_EDTA/test/test/tmp2'
    args.clean_flag = False
    args.threads = 56
    """

    # mkdir
    if args.tmp_work_dir is None:
        args.tmp_work_dir = get_file_dir(args.genome_seq_file) + "/tmp"
    
    mkdir(args.tmp_work_dir, True)

    rename_input_fasta = args.tmp_work_dir + "/input.fasta"
    rename_dict = {}
    
    with open(rename_input_fasta, 'w') as f:
        num = 0
        seq_dict = read_fasta_by_faidx(args.genome_seq_file)
        for i in seq_dict:
            num += 1
            new_seq_id = 'seq' + str(num)
            rename_dict[new_seq_id] = i
            f.write(">%s\n%s\n" % (new_seq_id, seq_dict[i].seq))
        
    
    # input
    if args.output_file is None:
        args.output_file = args.genome_seq_file + ".pfam.aa"

    # output
    transeq_output = rename_input_fasta + ".transeq"
    hmmscan_output = rename_input_fasta + ".hmmscan"

    transeq_cmd_string = 'transeq -frame 6 -sequence %s -outseq %s' % (rename_input_fasta, transeq_output)
    cmd_run(transeq_cmd_string)

    if args.threads <= 1:
        hmmscan_cmd_string = 'hmmscan --cpu 8 --domtblout %s %s %s' % (hmmscan_output, args.pfam_db_file, transeq_output)
        cmd_run(hmmscan_cmd_string)
    else:
        split_dir = args.tmp_work_dir + "/split"
        mkdir(split_dir, True)

        split_fasta(transeq_output, split_dir, 'split', contig_model=False, unit_num=6, file_size=None)

        args_list = []
        hmmscan_file_list = []
        for i in os.listdir(split_dir):
            if re.match('^split.*\.fa$', i):
                transeq_split_file = split_dir+"/"+i
                hmmscan_split_output = split_dir+"/"+i+".hmmscan"
                hmmscan_cmd_string = 'hmmscan --domtblout %s %s %s' % (hmmscan_split_output, args.pfam_db_file, transeq_split_file)
                args_list.append((hmmscan_cmd_string, None, 1, True))
                hmmscan_file_list.append(hmmscan_split_output)
        
        multiprocess_running(cmd_run, args_list, args.threads, silence=False)

        merge_file(hmmscan_file_list, hmmscan_output)

    output_dir = get_aa_seq(rename_input_fasta , transeq_output, hmmscan_output)

    for i in output_dir:
        strand = sorted(output_dir[i], key=lambda x:len(output_dir[i][x]), reverse=True)[0]
        output_dir[i] = output_dir[i][strand]

    with open(args.output_file, 'w') as f:
        for seq_id in output_dir:
            if len(output_dir[seq_id]) > 0:
                f.write(">%s\n%s\n" % (rename_dict[seq_id], output_dir[seq_id]))

    if args.clean_flag:
        rmdir(args.tmp_work_dir)



from copy import deepcopy
from typing import OrderedDict
from lib.common.os import cmd_run, mkdir, get_file_dir
from lib.common.genome.seq_base import read_fasta_by_faidx, write_fasta, FastaRecord
from lib.common.genome.genome_feature2 import Genome, genome_rename, write_gff_file, cds_judgment, Gene
import re


def read_given_gene_seq(args):
    given_cds_dict = None
    given_aa_dict = None

    if not args.cds_fasta_file is None:
        given_cds_dict = read_fasta_by_faidx(args.cds_fasta_file)
        if args.mode == 'ncbi':
            given_cds_dict_new = {}
            for i in given_cds_dict:
                record = given_cds_dict[i]
                protein_id = [j[1]
                              for j in record.qualifiers if j[0] == 'protein_id'][0]
                given_cds_dict_new[protein_id] = record
            given_cds_dict = given_cds_dict_new

        test_cds = True
    else:
        test_cds = False

    if not args.protein_fasta_file is None:
        given_aa_dict = read_fasta_by_faidx(args.protein_fasta_file)
        if args.mode == 'ncbi':
            given_aa_dict_new = {}
            for i in given_aa_dict:
                record = given_aa_dict[i]
                protein_id = [j[1]
                              for j in record.qualifiers if j[0] == 'protein_id'][0]
                given_aa_dict_new[protein_id] = record
            given_aa_dict = given_aa_dict_new
        test_pt = True
    else:
        test_pt = False

    return test_cds, test_pt, given_cds_dict, given_aa_dict


def GenomeFormater_main(args):
    mkdir(args.output_dir, True)

    # gunzip all file
    for file_tmp in ['genome_fasta_file', 'gff_file', 'protein_fasta_file', 'cds_fasta_file']:
        if not getattr(args, file_tmp) is None:
            file_path = getattr(args, file_tmp)
            if re.match(r'.*\.gz$', file_path):
                cmd_string = "gunzip %s" % file_path
                cmd_run(cmd_string, silence=True)
                file_path = re.sub(r'\.gz$', '', file_path)
                setattr(args, file_tmp, file_path)

    # if have genome and gff file, I will use it to extract cDNA, CDS and protein, other file will just use to check
    if args.gff_file is None:
        raise ValueError("must have gff file")

    if args.mode == 'ncbi':
        new_gff = args.gff_file + ".xyx"
        cmd_string = "grep \"exception=\" -v %s|grep \"^#\" -P -v |awk '{if ($5-$4 >= 0) print($_)}'> %s" % (args.gff_file, new_gff)
        cmd_run(cmd_string, cwd=get_file_dir(args.gff_file))
        args.gff_file = new_gff

    # create a genome object
    phyto_genome = Genome(genome_file=args.genome_fasta_file, gff_file=args.gff_file, cds_file=args.cds_fasta_file,
                          aa_file=args.protein_fasta_file)

    phyto_genome.genome_feature_parse()

    # bad gigascience
    if args.mode == 'normal' and 'mRNA' in phyto_genome.feature_dict:
        gene_dict = {}
        for mRNA_id in phyto_genome.feature_dict['mRNA']:
            mRNA_gf = phyto_genome.feature_dict['mRNA'][mRNA_id]
            gene_id = "gene." + mRNA_id
            gene_dict[gene_id] = Gene(
                id=gene_id, chr_loci=mRNA_gf.chr_loci, sub_features=[mRNA_gf])
            gene_dict[gene_id].type = 'gene'

        phyto_genome.feature_dict['gene'] = gene_dict

    # bad ncbi DDBJ gff
    if args.mode == 'ncbi':
        if phyto_genome.feature_dict['gene'][list(phyto_genome.feature_dict['gene'].keys())[0]].qualifiers['source'][0] == 'DDBJ':
            
            gene_dict = {}
            
            for g_id in phyto_genome.feature_dict['gene']:
                naked_cds_flag = False

                gene_gf = phyto_genome.feature_dict['gene'][g_id]
                if gene_gf.sub_features:
                    for cds in gene_gf.sub_features:
                        if cds.type == 'CDS':
                            naked_cds_flag = True
                        if cds.type == 'mRNA':
                            naked_cds_flag = False

                if naked_cds_flag:
                    mRNA_gf = deepcopy(gene_gf)

                    gene_gf.id = gene_gf.id + ".gene"
                    gene_gf.qualifiers['ID'][0] = gene_gf.qualifiers['ID'][0] + ".gene"
                    
                    mRNA_gf.type = 'mRNA'
                    mRNA_gf.qualifiers['Parent'] = [gene_gf.id]

                    gene_gf.sub_features = [mRNA_gf]

                gene_dict[gene_gf.id] = gene_gf
            
            phyto_genome.feature_dict['gene'] = gene_dict

    # filter non mRNA gene
    mRNA_gene_dict = OrderedDict()
    for g_id in phyto_genome.feature_dict['gene']:
        gf = phyto_genome.feature_dict['gene'][g_id]
        if gf.sub_features and gf.sub_features[0].type == 'mRNA':
            mRNA_gene_dict[g_id] = gf

    phyto_genome.feature_dict = {'gene':mRNA_gene_dict}

    # 
    if args.genome_fasta_file is not None:
        phyto_genome.genome_file_parse()
        phyto_genome.build_gene_sequence()

        # Compare builded seq and given seq
        test_cds, test_pt, given_cds_dict, given_aa_dict = read_given_gene_seq(
            args)

        # gene_number, match_cds_num, match_pt_num, match_cdna_num, cds_good_orf
        report_list = [0, 0, 0, 0]
        for gene_id in phyto_genome.feature_dict['gene']:
            gene_tmp = phyto_genome.feature_dict['gene'][gene_id]

            model_mRNA = [
                i for i in gene_tmp.sub_features if i.id == gene_tmp.model_mRNA_id][0]

            if args.mode == 'phytozome':
                model_id = model_mRNA.qualifiers['Name'][0]
            elif args.mode == 'ncbi':
                model_id = [i.qualifiers['protein_id'][0]
                            for i in model_mRNA.sub_features if i.type == 'CDS'][0]
            else:
                model_id = model_mRNA.qualifiers['ID'][0]

            report_list[0] += 1

            if test_cds and model_id in given_cds_dict and given_cds_dict[
                    model_id].seq.upper() == gene_tmp.model_cds_seq.upper():
                report_list[1] += 1

            if test_pt:
                if model_id in given_aa_dict:
                    if re.sub(r'\*', '', given_aa_dict[model_id].seq).upper() == gene_tmp.model_aa_seq.upper():
                        report_list[2] += 1
                    else:
                        print(re.sub(r'\*', '', given_aa_dict[model_id].seq))
                        print(gene_tmp.model_aa_seq)
                        print(model_id)

            if gene_tmp.model_cds_good_orf:
                report_list[3] += 1
            else:
                print(gene_id)

    else:
        phyto_genome.get_chromosome_from_gff()
        phyto_genome.build_gene_sequence()

        # read given seq
        test_cds, test_pt, given_cds_dict, given_aa_dict = read_given_gene_seq(
            args)

        # gene_number, find_cds_num, match_pt_num, cds_good_orf
        report_list = [0, 0, 0, 0]
        for gene_id in phyto_genome.feature_dict['gene']:
            gene_tmp = phyto_genome.feature_dict['gene'][gene_id]

            model_mRNA = [
                i for i in gene_tmp.sub_features if i.id == gene_tmp.model_mRNA_id][0]

            if args.mode == 'phytozome':
                model_id = model_mRNA.qualifiers['Name'][0]
            elif args.mode == 'ncbi':
                model_id = [i.qualifiers['protein_id']
                            for i in model_mRNA.sub_features if i.type == 'CDS'][0]
            else:
                model_id = model_mRNA.qualifiers['ID'][0]

            report_list[0] += 1

            if test_cds and model_id in given_cds_dict:
                report_list[1] += 1

                gene_tmp.model_cds_seq = given_cds_dict[model_id].seq
                good_orf, phase, aa_seq = cds_judgment(gene_tmp.model_cds_seq)

                if good_orf:
                    report_list[3] += 1
                # else:
                #     print(gene_id)

                gene_tmp.model_cds_good_orf = good_orf
                gene_tmp.model_aa_seq = aa_seq

                if test_pt:
                    if model_id in given_aa_dict:
                        if re.sub(r'\*', '', given_aa_dict[model_id].seq).upper() == gene_tmp.model_aa_seq.upper():
                            report_list[2] += 1
                        else:
                            print(
                                re.sub(r'\*', '', given_aa_dict[model_id].seq))
                            print(gene_tmp.model_aa_seq)
                            print(model_id)

            if test_pt and model_id in given_aa_dict:
                report_list[2] += 1
                aa_seq = re.sub(r'\*', '', given_aa_dict[model_id].seq)
                gene_tmp.model_aa_seq = aa_seq

    print("gene_num_in_gff: %d\nfind_cds_num: %d\nmatch_pt_num: %d\ngood_orf_cds: %d\n" %
          tuple(report_list))

    phyto_genome, chr_rename_dict, gene_rename_dict = genome_rename(
        phyto_genome, args.rename_prefix, args.keep_raw_contig_id)

    # write gff file
    new_gff_file = args.output_dir + "/" + args.rename_prefix + ".genome.gff3"
    gene_feature_dict = phyto_genome.feature_dict['gene']
    gf_list = [gene_feature_dict[i] for i in gene_feature_dict]
    write_gff_file(gf_list, new_gff_file)
    phyto_genome.gff_file = new_gff_file

    # write rename map
    rename_chr_map_file = args.output_dir + "/" + \
        args.rename_prefix + ".rename.chr.map"
    with open(rename_chr_map_file, 'w') as f:
        for raw_ctg_id in chr_rename_dict:
            chr_new_name = chr_rename_dict[raw_ctg_id]
            f.write("%s\t%s\n" % (raw_ctg_id, chr_new_name))

    rename_gene_map_file = args.output_dir + "/" + \
        args.rename_prefix + ".rename.gene.map"
    with open(rename_gene_map_file, 'w') as f:
        for raw_ctg_id in gene_rename_dict:
            chr_new_name = gene_rename_dict[raw_ctg_id]
            f.write("%s\t%s\n" % (raw_ctg_id, chr_new_name))

    # write genome file
    if args.genome_fasta_file is not None:
        new_genome_file = args.output_dir + "/" + args.rename_prefix + ".genome.fasta"

        genome_fasta_dict = read_fasta_by_faidx(args.genome_fasta_file)
        for new_chr_id in phyto_genome.chromosomes:
            old_id = phyto_genome.chromosomes[new_chr_id].old_id
            genome_fasta_dict[old_id].seqname = new_chr_id

        fasta_record_list = [genome_fasta_dict[i] for i in genome_fasta_dict]
        write_fasta(fasta_record_list, new_genome_file,
                    wrap_length=75, upper=True)
        phyto_genome.genome_file = new_genome_file

    # write protein, CDS, cDNA file
    cds_record_list = []
    cdna_record_list = []
    pt_record_list = []
    for gene_id in phyto_genome.feature_dict['gene']:
        gene_tmp = phyto_genome.feature_dict['gene'][gene_id]

        if hasattr(gene_tmp, 'model_cds_good_orf') and gene_tmp.model_cds_good_orf:
            if not gene_tmp.model_cds_seq == '':
                cds_record_list.append(FastaRecord(
                    gene_id, seq=gene_tmp.model_cds_seq))

            if not gene_tmp.model_cDNA_seq == '':
                cdna_record_list.append(FastaRecord(
                    gene_id, seq=gene_tmp.model_cDNA_seq))

            if not gene_tmp.model_aa_seq == '':
                pt_record_list.append(FastaRecord(
                    gene_id, seq=gene_tmp.model_aa_seq))

    new_cds_file = args.output_dir + "/" + \
        args.rename_prefix + ".gene_model.cds.fasta"
    write_fasta(cds_record_list, new_cds_file, wrap_length=75, upper=True)
    phyto_genome.cds_file = new_cds_file

    # new_cDNA_file = args.output_dir + "/" + args.rename_prefix + ".gene_model.cDNA.fasta"
    # write_fasta(cdna_record_list, new_cDNA_file, wrap_length=75)
    # phyto_genome.cDNA_file = new_cDNA_file

    new_pt_file = args.output_dir + "/" + \
        args.rename_prefix + ".gene_model.protein.fasta"
    write_fasta(pt_record_list, new_pt_file, wrap_length=75, upper=True)
    phyto_genome.aa_file = new_pt_file

    # phyto_genome.genome_file_parse()
    #
    # OUT = open(args.output_dir + "/" + args.rename_prefix + ".pyb", 'wb')
    # pickle.dump(phyto_genome, OUT)
    # OUT.close()

    # # gzip all file
    # for file_tmp in ['genome_fasta_file', 'gff_file', 'protein_fasta_file', 'cds_fasta_file', 'cdna_fasta_file']:
    #     if hasattr(args, file_tmp):
    #         file_path = getattr(args, file_tmp)
    #         cmd_string = "gzip %s" % file_path
    #         cmd_run(cmd_string, silence=True)
    #
    # for file_tmp in [new_genome_file, new_gff_file, new_cds_file, new_cDNA_file, new_pt_file]:
    #     cmd_string = "gzip %s" % file_path
    #     cmd_run(cmd_string, silence=True)


if __name__ == "__main__":

    class abc():
        pass

    args = abc()

    args.genome_fasta_file = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Pedicularis_cranolopha/Jinjianjun/Pcr.genome.1.0.fasta"
    args.gff_file = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Pedicularis_cranolopha/Jinjianjun/Pcr.genome.1.0.gff3"
    args.protein_fasta_file = None
    args.cds_fasta_file = None
    args.cdna_fasta_file = None
    args.keep_raw_contig_id = False
    args.rename_prefix = "T326817N0"
    args.mode = "normal"
    args.output_dir = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Pedicularis_cranolopha/Jinjianjun'

    GenomeFormater_main(args)

    class abc():
        pass

    args = abc()

    args.genome_fasta_file = "/lustre/home/xuyuxing/Work/PlantWGD/2.gene_filter/1.data_rename/tmp_152421/T152421N0.genome.fasta"
    args.gff_file = "/lustre/home/xuyuxing/Work/PlantWGD/2.gene_filter/1.data_rename/tmp_152421/T152421N0.genome.gff3"
    args.protein_fasta_file = None
    args.cds_fasta_file = None
    args.cdna_fasta_file = None
    args.keep_raw_contig_id = False
    args.rename_prefix = "T152421N0"
    args.mode = "ncbi"
    args.output_dir = '/lustre/home/xuyuxing/Work/PlantWGD/2.gene_filter/1.data_rename/T152421N0'

    GenomeFormater_main(args)



import drmaa
import os


def main():
    """
    Submit a job.
    Note, need file called sleeper.sh in current directory.
    """
    with drmaa.Session() as s:
        print('Creating job template')
        jt = s.createJobTemplate()
        jt.remoteCommand = os.path.join(os.getcwd(), 'sleeper.sh')
        jt.args = ['42', 'Simon says:']
        jt.joinFiles = True
        jt.outputPath = ":" + os.getcwd()

        jobid = s.runJob(jt)
        print('Your job has been submitted with ID %s' % jobid)

        print('Cleaning up')
        s.deleteJobTemplate(jt)


if __name__ == '__main__':
    main()



#!/usr/bin/python

##########################################################################################
##           Script to calculate 4-fold degenerate transversion rate                    ##
##           © 2014 Pascal Pucholt pascal.pucholt@slu.se                                ##
##########################################################################################


from Bio.Data.CodonTable import unambiguous_dna_by_id
from Bio.Seq import Seq
from Bio import AlignIO
from math import log
import argparse

parser = argparse.ArgumentParser(description='Calculate 4DTV')
parser.add_argument('-f', '--file', dest='file', default="1.aln",
                    help='filename of fasta alignment file')
parser.add_argument('-t', '--tab', action="store_const", dest='tab', const=1, default=0,
                    help='Output in one tab separted line')

args = parser.parse_args()


def altcodons(codon, table):
    """
    list codons that code for the same aminoacid / are also stop
    @param codon
    @table code table id
    @return list of codons
    """

    tab = unambiguous_dna_by_id[table]

    if codon in tab.stop_codons:
        return tab.stop_codons

    try:
        aa = tab.forward_table[codon]
    except:
        return []

    return [k for (k, v) in tab.forward_table.iteritems() if v == aa and k[0] == codon[0] and k[1] == codon[1]]


def degeneration(codon, table):
    """
    determine how many codons code for the same amino acid / are also stop
    as codon

    @param codon the codon
    @param table code table id
    @param the number of codons also coding for the aminoacid codon codes for
    """

    return len(altcodons(codon, table))


def isXdegenerated(x, codon, table):
    """
    determine if codon is x-fold degenerated

    @param codon the codon
    @param table code table id
    @param true if x <= the degeneration of the codon
    """

    return (x <= len(altcodons(codon, table)))


def degenerated_subseq(seq, x, table):
    """
    get a subsequence consisting of the x-fold degenerated codons only
    """

    data = ""
    for i in range(0, len(seq), 3):
        codon = seq[i:i + 3].tostring()
        if isXdegenerated(x, codon, table):
            data += codon
    return data


alignment = AlignIO.read(open(args.file), "fasta")
FD = alignment[:, 0:0]
FDx = FD
ID = 0
TS = 0
TV = 0
for i in xrange(0, alignment.get_alignment_length(), 3):
    al2 = alignment[:, i:(i + 3)]
    if (isXdegenerated(4, al2[0].seq.tostring(), 1)):
        if (al2[0, 0:2].seq.tostring() == al2[1, 0:2].seq.tostring()):
            FD = FD + al2

for i in xrange(0, FD.get_alignment_length(), 3):
    al2 = FD[:, i:(i + 3)]
    site = al2[:, 2:3]
    FDx = FDx + site
    n1 = site[0].seq.tostring()
    n2 = site[1].seq.tostring()
    if (n1 == n2):
        ID = ID + 1
    elif ((n1 == "A" or n1 == "G") and (n2 == "A" or n2 == "G")):
        TS = TS + 1
    elif ((n1 == "C" or n1 == "T") and (n2 == "C" or n2 == "T")):
        TS = TS + 1
    else:
        TV = TV + 1

LO = (float)(FD.get_alignment_length() / 3)
FDTV = TV / LO
FDTS = TS / LO
FDS = (TS + TV) / LO
if FDTV < 0.5:
    FDTVC = -0.5 * log(1 - 2 * FDTV)
else:
    FDTVC = 99.0
LO = int(LO)

if args.tab:
    print
    LO, "\t", ID, "\t", TS, "\t", TV, "\t", FDS, "\t", FDTS, "\t", FDTV, "\t", FDTVC
else:
    print
    "# of 4D sites: ", LO
    print
    "# identical sites: ", ID
    print
    "# ts sites: ", TS
    print
    "# tv sites: ", TV
    print
    "4DTV:", FDTV
    print
    "4DTS:", FDTS
    print
    "4DS:", FDS
    if FDS < 0.5:
        print
        "4DTVc:", FDTVC

    # print FDx



from lib.common.util import printer_list
from api.xuyuxing.resource.GO import Obo
from lib.common.os import multiprocess_running
from api.xuyuxing.file_parser.fileIO import tsv_file_parse, tsv_file_dict_parse
from lib.common.genome.seq_base import read_fasta_by_faidx
from api.xuyuxing.resource.uniprot import iterUniprotDB, extract_swiss_info
import pickle
import os
import re
from lib.common.genome.genome_feature2 import read_gff_file


def Arabidopsis_genome_function(work_dir, clean_data_fasta, clean_to_raw_id_map, araport_gff, uniprot_sprot_plants_db, uniprot_trembl_plants_db, kegg_overview, plantcyc_file, obo_file):

    ref_db_name = 'Araport'

    clean_seq_dict = read_fasta_by_faidx(clean_data_fasta)
    seq_annotation_dict = {i: {"dbxrefs": {}} for i in clean_seq_dict}

    tmp_info = tsv_file_parse(clean_to_raw_id_map)
    for i in tmp_info:
        clean_id, raw_id = tmp_info[i]

        if clean_id not in seq_annotation_dict:
            continue

        seq_annotation_dict[clean_id]['dbxrefs']['ref_id'] = raw_id

    # on araport11
    araport_gff = read_gff_file(araport_gff)

    tmp_info = {}
    for g_id in araport_gff['gene']:
        gf = araport_gff['gene'][g_id]
        # gene_name
        if 'symbol' in gf.qualifiers:
            gene_name = gf.qualifiers['symbol'][0]
        else:
            gene_name = ''
        # gene_alias
        if 'Alias' in gf.qualifiers:
            gene_alias = gf.qualifiers['Alias']
        else:
            gene_alias = []
        # full_name
        if 'full_name' in gf.qualifiers:
            full_name = gf.qualifiers['full_name'][0]
        else:
            full_name = ''
        # gene_note
        if 'Note' in gf.qualifiers:
            gene_note = gf.qualifiers['Note'][0]
        else:
            gene_note = ''
        
        tmp_info[g_id] = (gene_name, gene_alias, full_name, gene_note)
        
    for clean_id in seq_annotation_dict:
        raw_id = seq_annotation_dict[clean_id]['dbxrefs']['ref_id']
        if raw_id in tmp_info:
            seq_annotation_dict[clean_id]['gff_info'] = tmp_info[raw_id]


    # on uniprot

    uniprot_map_file = work_dir+"/uniprot.map"

    if not os.path.exists(uniprot_map_file):
        ref_id_list = set()
        with open(work_dir+"/uniprot.map", 'w') as f:
            num = 0

            for db_tmp in [uniprot_sprot_plants_db, uniprot_trembl_plants_db]:

                for id_tmp, obj in iterUniprotDB(db_tmp):
                    num += 1
                    print(num)

                    dbxref_dict = {}
                    for i in obj.dbxrefs:
                        db_name, db_record = i.split(":", 1)
                        if db_name not in dbxref_dict:
                            dbxref_dict[db_name] = []
                        dbxref_dict[db_name].append(db_record)

                    if ref_db_name in dbxref_dict:
                        ref_id = dbxref_dict[ref_db_name][0]
                        if ref_id not in ref_id_list:
                            ref_id_list = ref_id_list | set([ref_id])
                            f.write("%s\t%s\n" % (ref_id, id_tmp))

    tmp_info = tsv_file_parse(uniprot_map_file)
    ref_dict = {}
    for i in tmp_info:
        ref_id, uniprot_id = tmp_info[i]
        ref_dict[ref_id] = uniprot_id

    for clean_id in seq_annotation_dict:
        ref_id = seq_annotation_dict[clean_id]['dbxrefs']['ref_id']

        if ref_id in ref_dict:
            seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] = ref_dict[ref_id]
        else:
            seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] = None

    print("There are %d genes, %d have uniprot id, %d failed to find" % (len(seq_annotation_dict), len(
        [i for i in seq_annotation_dict if seq_annotation_dict[i]['dbxrefs']['uniprot_id']]), len([i for i in seq_annotation_dict if seq_annotation_dict[i]['dbxrefs']['uniprot_id'] is None])))

    uniq_id_list = [seq_annotation_dict[i]['dbxrefs']['uniprot_id']
                    for i in seq_annotation_dict if not seq_annotation_dict[i]['dbxrefs']['uniprot_id'] is None]

    sprot_dict = extract_swiss_info(uniq_id_list, uniprot_sprot_plants_db)
    trembl_dict = extract_swiss_info(uniq_id_list, uniprot_trembl_plants_db)

    for clean_id in seq_annotation_dict:
        uniprot_id = seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id']

        if uniprot_id:
            if uniprot_id in sprot_dict:
                uni_obj = sprot_dict[uniprot_id]
            elif uniprot_id in trembl_dict:
                uni_obj = trembl_dict[uniprot_id]
            else:
                raise ValueError("error 1")

            # # gene name
            # fullName = uni_obj.description
            # gene_name = uni_obj.annotations['gene_name_primary']
            # alias = uni_obj.annotations['gene_name_synonym']
            # keywords = uni_obj.annotations['keywords']
            # comment_function = uni_obj.annotations['comment_function']

            # db covert
            dbxref_dict = {}
            for i in uni_obj.dbxrefs:
                db_name, db_record = i.split(":", 1)
                if db_name not in dbxref_dict:
                    dbxref_dict[db_name] = []
                dbxref_dict[db_name].append(db_record)

            for i in dbxref_dict:
                seq_annotation_dict[clean_id]['dbxrefs'][i] = dbxref_dict[i]

    for key_word in ['uniprot_id', 'BioCyc', 'GO', 'KEGG', 'Pfam', 'PubMed', 'RefSeq']:
        all_gene = len(seq_annotation_dict)
        get_num = len([i for i in seq_annotation_dict if seq_annotation_dict[i]['dbxrefs']['uniprot_id']
                       and key_word in seq_annotation_dict[i]['dbxrefs'] and len(seq_annotation_dict[i]['dbxrefs'][key_word]) > 0])
        failed_num = all_gene - get_num
        print("There are %d genes, %d have %s, %d failed to find" %
              (all_gene, get_num, key_word, failed_num))

    # add kegg
    kegg_info_dict = tsv_file_parse(kegg_overview, key_col=1)

    key_word = 'KEGG'
    for clean_id in seq_annotation_dict:
        if seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] and key_word in seq_annotation_dict[clean_id]['dbxrefs'] and len(seq_annotation_dict[clean_id]['dbxrefs'][key_word]) > 0:
            kegg_id = seq_annotation_dict[clean_id]['dbxrefs'][key_word][0]
            if kegg_id in kegg_info_dict:
                kegg_info = kegg_info_dict[kegg_id]
                kg_id, ncbi_id, pw_id, pw_info, md_id, md_info, ko_id, ko_info = kegg_info
                seq_annotation_dict[clean_id]['KEGG'] = (
                    pw_id, pw_info, md_id, md_info, ko_id, ko_info)

    # add GO
    ontology = Obo(obo_file)

    key_word = 'GO'
    all_go_list = []
    for clean_id in seq_annotation_dict:

        if seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] and key_word in seq_annotation_dict[clean_id]['dbxrefs'] and len(seq_annotation_dict[clean_id]['dbxrefs'][key_word]) > 0:
            go_id_list = seq_annotation_dict[clean_id]['dbxrefs'][key_word]

            go_info_dict = {}
            for go_id in go_id_list:
                go_info = ontology.get(go_id)
                if go_info:
                    go_info_dict[go_info['id']] = go_info
                    all_go_list.append(go_id)

            seq_annotation_dict[clean_id]['GO'] = go_info_dict

    all_go_list = list(set(all_go_list))

    def get_all_parents(go_id):
        parent_list = [i[0] for i in ontology.find_parents(go_id, expand=True)]
        parent_list.append(go_id)
        return list(set(parent_list))

    go_parent_dict = multiprocess_running(
        get_all_parents, [(i,) for i in all_go_list], 56, args_id_list=all_go_list)

    for clean_id in seq_annotation_dict:
        if seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] and key_word in seq_annotation_dict[clean_id]['dbxrefs'] and len(seq_annotation_dict[clean_id]['dbxrefs'][key_word]) > 0:
            go_id_list = seq_annotation_dict[clean_id]['dbxrefs'][key_word]

            all_parent_list = []
            for i in go_id_list:
                if i in go_parent_dict:
                    all_parent_list.extend(go_parent_dict[i]['output'])

            all_parent_list = list(set(all_parent_list))

            seq_annotation_dict[clean_id]['GO_parent'] = all_parent_list

    # add Plantcyc
    tmp_info = tsv_file_parse(plantcyc_file)
    cyc_info_dict = {}
    for i in tmp_info:
        pw_id, pw_info, ra_id, ec, p_id, p_name, g_id, g_name = tmp_info[i]
        if g_id not in cyc_info_dict:
            cyc_info_dict[g_id] = []
        cyc_info_dict[g_id].append((pw_id, pw_info))

    cyc_info_dict2 = {}
    for i in tmp_info:
        pw_id, pw_info, ra_id, ec, p_id, p_name, g_id, g_name = tmp_info[i]
        if g_name not in cyc_info_dict2:
            cyc_info_dict2[g_name] = []
        cyc_info_dict2[g_name].append((pw_id, pw_info))

    key_word = 'BioCyc'
    num = 0
    num2 = 0
    for clean_id in seq_annotation_dict:
        if seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] and key_word in seq_annotation_dict[clean_id]['dbxrefs'] and len(seq_annotation_dict[clean_id]['dbxrefs'][key_word]) > 0:
            cyc_id = seq_annotation_dict[clean_id]['dbxrefs'][key_word][0]
            cyc_id = cyc_id.split(":")[1]
            cyc_id = cyc_id.split("-")[0]
            num2 += 1
            if cyc_id in cyc_info_dict:
                cyc_list = cyc_info_dict[cyc_id]
                seq_annotation_dict[clean_id]['Plantcyc'] = cyc_list
            elif cyc_id in cyc_info_dict2:
                cyc_list = cyc_info_dict2[cyc_id]
                seq_annotation_dict[clean_id]['Plantcyc'] = cyc_list
            else:
                # print(cyc_id)
                num += 1

    # get output

    with open(work_dir+"/all_annotation.tsv", 'w') as f:
        head_list = ["ref_id", "Gene Name", "Alias", "Full Name", "key_words", "Computational Description", "PubMed_number",
                     "KEGG pathway(id)", "KEGG pathway", "KEGG module(id)", "KEGG module",  "KEGG KO(id)", "KEGG KO", "GO(id)", "GO_parent", "GO", "Plantcyc(id)", "Plantcyc",  "clean_id"]

        head_string = printer_list(head_list, sep='\t')
        f.write(head_string + "\n")

        data_dict = {}

        for clean_id in seq_annotation_dict:
            uniprot_id = seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id']
            ref_id = seq_annotation_dict[clean_id]['dbxrefs']['ref_id']

            if uniprot_id in sprot_dict:
                uni_obj = sprot_dict[uniprot_id]
            elif uniprot_id in trembl_dict:
                uni_obj = trembl_dict[uniprot_id]
            else:
                gene_name, fullName, keywords, comment_function, pw_id, pw_info, md_id, md_info, ko_id, ko_info, go_id_string, GO_parent_string, go_des_string, cyc_id_string, cyc_des_string = [
                    ""]*15
                alias = []
                keywords = []
                PubMed_number = 0

            # from uniprot
            fullName = uni_obj.description

            if 'gene_name_primary' in uni_obj.annotations:
                gene_name = uni_obj.annotations['gene_name_primary']
            else:
                gene_name = ""

            if 'gene_name_synonym' in uni_obj.annotations:
                alias = uni_obj.annotations['gene_name_synonym']
            else:
                alias = []

            if 'keywords' in uni_obj.annotations:
                keywords = uni_obj.annotations['keywords']
            else:
                keywords = []

            if 'comment_function' in uni_obj.annotations:
                comment_function = uni_obj.annotations['comment_function']
                comment_function = printer_list(comment_function, '; ')
            else:
                comment_function = ""

            if 'references' in uni_obj.annotations:
                PubMed_list = uni_obj.annotations['references']
                PubMed_number = len(PubMed_list)
            else:
                PubMed_number = 0

            # from other db
            # kegg
            if 'KEGG' in seq_annotation_dict[clean_id]:
                pw_id, pw_info, md_id, md_info, ko_id, ko_info = seq_annotation_dict[
                    clean_id]['KEGG']
                tmp_tuple = []
                for i in pw_id, pw_info, md_id, md_info, ko_id, ko_info:
                    if i == 'None':
                        tmp_tuple.append("")
                    else:
                        i = i.replace(
                            ' - Arabidopsis thaliana (thale cress)', '')
                        tmp_tuple.append(i)

                pw_id, pw_info, md_id, md_info, ko_id, ko_info = tmp_tuple
            else:
                pw_id, pw_info, md_id, md_info, ko_id, ko_info = [""] * 6

            # go
            if 'GO' in seq_annotation_dict[clean_id]:
                go_info_dict = seq_annotation_dict[clean_id]['GO']

                go_list = list(go_info_dict.keys())
                go_id_string = printer_list(go_list, '; ')

                go_des = [go_info_dict[i]['name'] for i in go_list]
                go_des_string = printer_list(go_des, '; ')

                GO_parent_list = seq_annotation_dict[clean_id]['GO_parent']
                GO_parent_string = printer_list(GO_parent_list, '; ')

            else:
                go_id_string, go_des_string, GO_parent_string = [""] * 3

            # Plantcyc
            if 'Plantcyc' in seq_annotation_dict[clean_id]:
                cyc_list = seq_annotation_dict[clean_id]['Plantcyc']
                cyc_id_list = []
                cyc_des_list = []

                for i in cyc_list:
                    cyc_id_list.append(i[0])
                for i in cyc_list:
                    cyc_des_list.append(i[1])

                cyc_id_string = printer_list(cyc_id_list, '; ')
                cyc_des_string = printer_list(cyc_des_list, '; ')

            else:
                cyc_id_string, cyc_des_string = [""] * 2

            # 
            if 'gff_info' in seq_annotation_dict[clean_id]:
                gff_gene_name, gff_gene_alias, gff_full_name, gff_gene_note = seq_annotation_dict[clean_id]['gff_info']

                alias.extend(gff_gene_alias)
                alias = list(set(alias))

                keywords.extend([gff_gene_note])
                keywords = list(set(keywords))

                if gene_name == "" and gff_gene_name != "":
                    gene_name = gff_gene_name

                if fullName == "" and gff_full_name != "":
                    fullName = gff_full_name

            if '' in alias:
                alias.remove('')
            alias = printer_list(alias, '; ')

            if '' in keywords:
                keywords.remove('')
            keywords = printer_list(keywords, '; ')


            printer_string = printer_list([ref_id, gene_name, alias, fullName, keywords, comment_function, PubMed_number, pw_id, pw_info, md_id,
                                           md_info, ko_id, ko_info, go_id_string, GO_parent_string, go_des_string, cyc_id_string, cyc_des_string, clean_id], "\t")

            f.write(printer_string+'\n')

            data_dict[clean_id] = (ref_id, gene_name, alias, fullName, keywords, comment_function, PubMed_number, pw_id, pw_info, md_id,
                                   md_info, ko_id, ko_info, go_id_string, GO_parent_string, go_des_string, cyc_id_string, cyc_des_string, clean_id)

    OUT = open(work_dir + "/all_annotation.pyb", 'wb')
    pickle.dump(data_dict, OUT)
    OUT.close()


def Oryza_genome_function(work_dir, clean_data_fasta, clean_to_raw_id_map,blast_id_map, uniprot_sprot_plants_db, uniprot_trembl_plants_db, kegg_overview, plantcyc_file, obo_file):

    clean_seq_dict = read_fasta_by_faidx(clean_data_fasta)
    seq_annotation_dict = {i: {"dbxrefs": {}} for i in clean_seq_dict}

    tmp_info = tsv_file_parse(clean_to_raw_id_map)
    for i in tmp_info:
        clean_id, raw_id = tmp_info[i]

        if clean_id not in seq_annotation_dict:
            continue

        seq_annotation_dict[clean_id]['dbxrefs']['ref_id'] = raw_id

    tsv_info = tsv_file_parse(RAP_MSU_table_file)

    MSU_to_RAP_dict = {}
    for i in tsv_info:
        RAP_id, MSU_id = tsv_info[i]

        for j in MSU_id.split(","):
            if j == 'None':
                continue
            else:
                j = j.split(".")[0]
                MSU_to_RAP_dict[j] = RAP_id

    #  RAP
    for clean_id in seq_annotation_dict:
        MSU_id = seq_annotation_dict[clean_id]['dbxrefs']['ref_id']
        if MSU_id in MSU_to_RAP_dict:
            RAP_id = MSU_to_RAP_dict[MSU_id]
            seq_annotation_dict[clean_id]['dbxrefs']['RAP_id'] = RAP_id

    # from RAP
    RAP_info_dict = tsv_file_dict_parse(RAP_tsv_file, key_col='Locus_ID')

    for clean_id in seq_annotation_dict:
        if not 'RAP_id' in seq_annotation_dict[clean_id]['dbxrefs']:
            continue

        RAP_id = seq_annotation_dict[clean_id]['dbxrefs']['RAP_id']

        if RAP_id not in RAP_info_dict:
            continue

        seq_annotation_dict[clean_id]['raw_tsv'] = {}

        RAP_info = RAP_info_dict[RAP_id]

        # gene_name
        gene_name = RAP_info['CGSNL Gene Name']

        # alias
        alias = RAP_info['RAP-DB Gene Symbol Synonym(s)'].split(", ")
        alias.extend(RAP_info['RAP-DB Gene Name Synonym(s)'].split(", "))
        alias.extend(RAP_info['CGSNL Gene Symbol'].split(", "))
        alias.extend(RAP_info['CGSNL Gene Name'].split(", "))
        alias.extend(RAP_info['Oryzabase Gene Symbol Synonym(s)'].split(", "))
        alias.extend(RAP_info['Oryzabase Gene Name Synonym(s)'].split(", "))

        alias = list(set(alias))

        # Full Name
        Full_Name = RAP_info['Description']

        # GO
        GO_list = re.findall('GO:.......', RAP_info['GO'])

        # PMID
        PMID = RAP_info['Literature PMID'].split(", ")

        seq_annotation_dict[clean_id]['raw_tsv'] = {
            'gene_name': gene_name,
            'alias': alias,
            'Full_Name': Full_Name,
            'GO_list': GO_list,
            'PMID': PMID,
        }

    # uniprot

    uniprot_map_file = work_dir+"/uniprot.map"

    if not os.path.exists(uniprot_map_file):
        RAP_or_MSU_id = []
        for clean_id in seq_annotation_dict:
            MSU_id = seq_annotation_dict[clean_id]['dbxrefs']['ref_id']
            if MSU_id != 'None':
                RAP_or_MSU_id.append(MSU_id)
            if 'RAP_id' in seq_annotation_dict[clean_id]['dbxrefs']:
                RAP_id = seq_annotation_dict[clean_id]['dbxrefs']['RAP_id']
                if RAP_id != 'None':
                    RAP_or_MSU_id.append(RAP_id)

        osa_all_gene_list = set(RAP_or_MSU_id)

        with open(work_dir+"/uniprot.map", 'w') as f:
            num = 0

            for db_tmp in [uniprot_sprot_plants_db, uniprot_trembl_plants_db]:

                for id_tmp, obj in iterUniprotDB(db_tmp):
                    num += 1
                    print(num)

                    dbxref_dict = {}
                    for i in obj.dbxrefs:
                        db_name, db_record = i.split(":", 1)
                        if db_name not in dbxref_dict:
                            dbxref_dict[db_name] = []
                        dbxref_dict[db_name].append(db_record)

                    if 'gene_name_ordered locus' in obj.annotations:
                        ref_ids = obj.annotations['gene_name_ordered locus']
                        ref_id_get = set(ref_ids) & osa_all_gene_list
                        if len(ref_id_get) > 0:
                            for i in ref_id_get:
                                f.write("%s\t%s\n" % (i, id_tmp))

    tmp_info = tsv_file_parse(uniprot_map_file)
    ref_dict = {}
    for i in tmp_info:
        ref_id, uniprot_id = tmp_info[i]
        ref_dict[ref_id] = uniprot_id

    for clean_id in seq_annotation_dict:
        ref_id = seq_annotation_dict[clean_id]['dbxrefs']['ref_id']

        if ref_id in ref_dict:
            seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] = ref_dict[ref_id]
        else:
            seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] = None

    # add blast results
    tmp_info = tsv_file_parse(blast_id_map)
    blast_dict = {}
    for i in tmp_info:
        clean_id, uniprot_id = tmp_info[i]
        blast_dict[clean_id] = uniprot_id

    for i in seq_annotation_dict:
        if seq_annotation_dict[i]['dbxrefs']['uniprot_id'] is None:
            if i in blast_dict:
                seq_annotation_dict[i]['dbxrefs']['uniprot_id'] = blast_dict[i]

    print("There are %d genes, %d have uniprot id, %d failed to find" % (len(seq_annotation_dict), len(
        [i for i in seq_annotation_dict if seq_annotation_dict[i]['dbxrefs']['uniprot_id']]), len([i for i in seq_annotation_dict if seq_annotation_dict[i]['dbxrefs']['uniprot_id'] is None])))


    uniq_id_list = [seq_annotation_dict[i]['dbxrefs']['uniprot_id']
                    for i in seq_annotation_dict if not seq_annotation_dict[i]['dbxrefs']['uniprot_id'] is None]

    sprot_dict = extract_swiss_info(uniq_id_list, uniprot_sprot_plants_db)
    trembl_dict = extract_swiss_info(uniq_id_list, uniprot_trembl_plants_db)

    for clean_id in seq_annotation_dict:
        uniprot_id = seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id']

        if uniprot_id:
            if uniprot_id in sprot_dict:
                uni_obj = sprot_dict[uniprot_id]
            elif uniprot_id in trembl_dict:
                uni_obj = trembl_dict[uniprot_id]
            else:
                raise ValueError("error 1")

            # # gene name
            # fullName = uni_obj.description
            # gene_name = uni_obj.annotations['gene_name_primary']
            # alias = uni_obj.annotations['gene_name_synonym']
            # keywords = uni_obj.annotations['keywords']
            # comment_function = uni_obj.annotations['comment_function']

            # db covert
            dbxref_dict = {}
            for i in uni_obj.dbxrefs:
                db_name, db_record = i.split(":", 1)
                if db_name not in dbxref_dict:
                    dbxref_dict[db_name] = []
                dbxref_dict[db_name].append(db_record)

            for i in dbxref_dict:
                seq_annotation_dict[clean_id]['dbxrefs'][i] = dbxref_dict[i]

    for key_word in ['uniprot_id', 'BioCyc', 'GO', 'KEGG', 'Pfam', 'PubMed', 'RefSeq']:
        all_gene = len(seq_annotation_dict)
        get_num = len([i for i in seq_annotation_dict if seq_annotation_dict[i]['dbxrefs']['uniprot_id']
                       and key_word in seq_annotation_dict[i]['dbxrefs'] and len(seq_annotation_dict[i]['dbxrefs'][key_word]) > 0])
        failed_num = all_gene - get_num
        print("There are %d genes, %d have %s, %d failed to find" %
              (all_gene, get_num, key_word, failed_num))

    # add kegg
    kegg_info_dict = tsv_file_parse(kegg_overview, key_col=1)

    key_word = 'KEGG'
    for clean_id in seq_annotation_dict:
        if seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] and key_word in seq_annotation_dict[clean_id]['dbxrefs'] and len(seq_annotation_dict[clean_id]['dbxrefs'][key_word]) > 0:
            kegg_id = seq_annotation_dict[clean_id]['dbxrefs'][key_word][0]
            if kegg_id in kegg_info_dict:
                kegg_info = kegg_info_dict[kegg_id]
                kg_id, ncbi_id, pw_id, pw_info, md_id, md_info, ko_id, ko_info = kegg_info
                seq_annotation_dict[clean_id]['KEGG'] = (
                    pw_id, pw_info, md_id, md_info, ko_id, ko_info)

    # add GO
    ontology = Obo(obo_file)

    key_word = 'GO'
    all_go_list = []
    for clean_id in seq_annotation_dict:

        if seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] and key_word in seq_annotation_dict[clean_id]['dbxrefs'] and len(seq_annotation_dict[clean_id]['dbxrefs'][key_word]) > 0:
            go_id_list = seq_annotation_dict[clean_id]['dbxrefs'][key_word]

            go_info_dict = {}
            for go_id in go_id_list:
                go_info = ontology.get(go_id)
                if go_info:
                    go_info_dict[go_info['id']] = go_info
                    all_go_list.append(go_id)

            seq_annotation_dict[clean_id]['GO'] = go_info_dict

    all_go_list = list(set(all_go_list))

    def get_all_parents(go_id):
        parent_list = [i[0] for i in ontology.find_parents(go_id, expand=True)]
        parent_list.append(go_id)
        return list(set(parent_list))

    go_parent_dict = multiprocess_running(
        get_all_parents, [(i,) for i in all_go_list], 56, args_id_list=all_go_list)

    for clean_id in seq_annotation_dict:
        if seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] and key_word in seq_annotation_dict[clean_id]['dbxrefs'] and len(seq_annotation_dict[clean_id]['dbxrefs'][key_word]) > 0:
            go_id_list = seq_annotation_dict[clean_id]['dbxrefs'][key_word]

            all_parent_list = []
            for i in go_id_list:
                if i in go_parent_dict:
                    all_parent_list.extend(go_parent_dict[i]['output'])

            all_parent_list = list(set(all_parent_list))

            seq_annotation_dict[clean_id]['GO_parent'] = all_parent_list

    # add Plantcyc
    tmp_info = tsv_file_parse(plantcyc_file)
    cyc_info_dict = {}
    for i in tmp_info:
        pw_id, pw_info, ra_id, ec, p_id, p_name, g_id, g_name = tmp_info[i]
        if g_id not in cyc_info_dict:
            cyc_info_dict[g_id] = []
        cyc_info_dict[g_id].append((pw_id, pw_info))

    cyc_info_dict2 = {}
    for i in tmp_info:
        pw_id, pw_info, ra_id, ec, p_id, p_name, g_id, g_name = tmp_info[i]
        if g_name not in cyc_info_dict2:
            cyc_info_dict2[g_name] = []
        cyc_info_dict2[g_name].append((pw_id, pw_info))

    key_word = 'BioCyc'
    num = 0
    num2 = 0
    for clean_id in seq_annotation_dict:
        if seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id'] and key_word in seq_annotation_dict[clean_id]['dbxrefs'] and len(seq_annotation_dict[clean_id]['dbxrefs'][key_word]) > 0:
            cyc_id = seq_annotation_dict[clean_id]['dbxrefs'][key_word][0]
            cyc_id = cyc_id.split(":")[1]
            cyc_id = cyc_id.split("-")[0]
            num2 += 1
            if cyc_id in cyc_info_dict:
                cyc_list = cyc_info_dict[cyc_id]
                seq_annotation_dict[clean_id]['Plantcyc'] = cyc_list
            elif cyc_id in cyc_info_dict2:
                cyc_list = cyc_info_dict2[cyc_id]
                seq_annotation_dict[clean_id]['Plantcyc'] = cyc_list
            else:
                # print(cyc_id)
                num += 1

    # get output

    with open(work_dir+"/all_annotation.tsv", 'w') as f:
        head_list = ["ref_id", "Gene Name", "Alias", "Full Name", "key_words", "Computational Description", "PubMed_number",
                     "KEGG pathway(id)", "KEGG pathway", "KEGG module(id)", "KEGG module",  "KEGG KO(id)", "KEGG KO", "GO(id)", "GO_parent", "GO", "Plantcyc(id)", "Plantcyc",  "clean_id"]

        head_string = printer_list(head_list, sep='\t')
        f.write(head_string + "\n")

        data_dict = {}

        for clean_id in seq_annotation_dict:
            uniprot_id = seq_annotation_dict[clean_id]['dbxrefs']['uniprot_id']
            ref_id = seq_annotation_dict[clean_id]['dbxrefs']['ref_id']

            if uniprot_id in sprot_dict:
                uni_obj = sprot_dict[uniprot_id]
            elif uniprot_id in trembl_dict:
                uni_obj = trembl_dict[uniprot_id]
            else:
                gene_name, fullName, keywords, comment_function, PubMed_number, pw_id, pw_info, md_id, md_info, ko_id, ko_info, go_id_string, GO_parent_string, go_des_string, cyc_id_string, cyc_des_string = [
                    ""]*16
                alias = []

            # from uniprot
            fullName = uni_obj.description

            if 'gene_name_primary' in uni_obj.annotations:
                gene_name = uni_obj.annotations['gene_name_primary']
            else:
                gene_name = ""

            if 'gene_name_synonym' in uni_obj.annotations:
                alias = uni_obj.annotations['gene_name_synonym']
            else:
                alias = []

            if 'keywords' in uni_obj.annotations:
                keywords = uni_obj.annotations['keywords']
                keywords = printer_list(keywords, '; ')
            else:
                keywords = ""

            if 'comment_function' in uni_obj.annotations:
                comment_function = uni_obj.annotations['comment_function']
                comment_function = printer_list(comment_function, '; ')
            else:
                comment_function = ""

            if 'references' in uni_obj.annotations:
                PubMed_list = uni_obj.annotations['references']
                PubMed_number = len(PubMed_list)
            else:
                PubMed_number = ""

            # from other db
            # kegg
            if 'KEGG' in seq_annotation_dict[clean_id]:
                pw_id, pw_info, md_id, md_info, ko_id, ko_info = seq_annotation_dict[
                    clean_id]['KEGG']
                tmp_tuple = []
                for i in pw_id, pw_info, md_id, md_info, ko_id, ko_info:
                    if i == 'None':
                        tmp_tuple.append("")
                    else:
                        i = i.replace(
                            ' - Arabidopsis thaliana (thale cress)', '')
                        tmp_tuple.append(i)

                pw_id, pw_info, md_id, md_info, ko_id, ko_info = tmp_tuple
            else:
                pw_id, pw_info, md_id, md_info, ko_id, ko_info = [""] * 6

            # go
            if 'GO' in seq_annotation_dict[clean_id]:
                go_info_dict = seq_annotation_dict[clean_id]['GO']

                go_list = list(go_info_dict.keys())
                go_id_string = printer_list(go_list, '; ')

                go_des = [go_info_dict[i]['name'] for i in go_list]
                go_des_string = printer_list(go_des, '; ')

                GO_parent_list = seq_annotation_dict[clean_id]['GO_parent']
                GO_parent_string = printer_list(GO_parent_list, '; ')

            else:
                go_id_string, go_des_string, GO_parent_string = [""] * 3

            # Plantcyc
            if 'Plantcyc' in seq_annotation_dict[clean_id]:
                cyc_list = seq_annotation_dict[clean_id]['Plantcyc']
                cyc_id_list = []
                cyc_des_list = []

                for i in cyc_list:
                    cyc_id_list.append(i[0])
                for i in cyc_list:
                    cyc_des_list.append(i[1])

                cyc_id_string = printer_list(cyc_id_list, '; ')
                cyc_des_string = printer_list(cyc_des_list, '; ')

            else:
                cyc_id_string, cyc_des_string = [""] * 2

            if 'raw_tsv' in seq_annotation_dict[clean_id]:
                if 'alias' in seq_annotation_dict[clean_id]['raw_tsv']:
                    alias.extend(seq_annotation_dict[clean_id]['raw_tsv']['alias'])
                    alias = list(set(alias))

                if gene_name == "" and 'gene_name' in seq_annotation_dict[clean_id]['raw_tsv']:
                    gene_name = seq_annotation_dict[clean_id]['raw_tsv']['gene_name']

                if fullName == "" and 'Full_Name' in seq_annotation_dict[clean_id]['raw_tsv']:
                    fullName = seq_annotation_dict[clean_id]['raw_tsv']['Full_Name']

            if '' in alias:
                alias.remove('')
            alias = printer_list(alias, '; ')


            printer_string = printer_list([ref_id, gene_name, alias, fullName, keywords, comment_function, PubMed_number, pw_id, pw_info, md_id,
                                           md_info, ko_id, ko_info, go_id_string, GO_parent_string, go_des_string, cyc_id_string, cyc_des_string, clean_id], "\t")

            f.write(printer_string+'\n')

            data_dict[clean_id] = (ref_id, gene_name, alias, fullName, keywords, comment_function, PubMed_number, pw_id, pw_info, md_id,
                                   md_info, ko_id, ko_info, go_id_string, GO_parent_string, go_des_string, cyc_id_string, cyc_des_string, clean_id)

    OUT = open(work_dir + "/all_annotation.pyb", 'wb')
    pickle.dump(data_dict, OUT)
    OUT.close()


if __name__ == "__main__":
    # Ath
    work_dir = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/function"
    clean_data_fasta = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/T3702N0.gene_model.protein.fasta'
    clean_interpro = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/T3702N0.gene_model.protein.interpro.tsv'
    clean_to_raw_id_map = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/function/id.map'


    araport_gff = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/function/Araport11_GFF3_genes_transposons.201606.gff'

    # uniprot
    uniprot_sprot_plants_db = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_sprot_plants.db'
    uniprot_trembl_plants_db = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_trembl_plants.db'

    # kegg
    kegg_overview = '/lustre/home/xuyuxing/Database/kegg/ath/ath.overview.tsv'

    # plantcyc
    plantcyc_file = '/lustre/home/xuyuxing/Database/plantcyc/aracyc_pathways.20180702'

    # go
    obo_file = "/lustre/home/xuyuxing/Database/GO/go.obo"

    Arabidopsis_genome_function(work_dir, clean_data_fasta, clean_to_raw_id_map,
                                uniprot_sprot_plants_db, uniprot_trembl_plants_db, kegg_overview, plantcyc_file, obo_file)

    # Osa

    work_dir = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Oryza_sativa/function"
    clean_data_fasta = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Oryza_sativa/T4530N0.gene_model.protein.fasta'
    clean_interpro = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Oryza_sativa/T4530N0.gene_model.protein.interpro.tsv'
    clean_to_raw_id_map = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Oryza_sativa/function/id.map'
    

    # RAP
    RAP_MSU_table_file = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Oryza_sativa/function/RAP-MSU_2020-09-09.txt"
    RAP_tsv_file = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Oryza_sativa/function/IRGSP-1.0_representative_annotation_2020-09-09.tsv"

    # uniprot
    uniprot_sprot_plants_db = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_sprot_plants.db'
    uniprot_trembl_plants_db = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_trembl_plants.db'
    blast_id_map = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Oryza_sativa/function/osa_vs_uniprot.idmap'

    # kegg
    kegg_overview = '/lustre/home/xuyuxing/Database/kegg/osa/osa.overview.tsv'

    # plantcyc
    plantcyc_file = '/lustre/home/xuyuxing/Database/plantcyc/oryzacyc_pathways.20180702'

    # go
    obo_file = "/lustre/home/xuyuxing/Database/GO/go.obo"

    Arabidopsis_genome_function(work_dir, clean_data_fasta, clean_to_raw_id_map, ref_db_name,
                                uniprot_sprot_plants_db, uniprot_trembl_plants_db, kegg_overview, plantcyc_file, obo_file)



from lib.xuyuxing.structure.dcd_file import get_a_frame_from_dcd_to_pdb, atom_distance, get_RMSD, get_RE_RMSD
from api.xuyuxing.file_parser.fileIO import tsv_file_parse
from lib.common.util import printer_list

if __name__ == '__main__':
    import argparse

    ###### argument parse
    parser = argparse.ArgumentParser(
        prog='StruTools',
    )

    subparsers = parser.add_subparsers(title='subcommands', dest="subcommand_name")

    # argparse for Dcd2pdb
    parser_a = subparsers.add_parser('Dcd2pdb',
                                     help='get some or all frame coord from dcd file to pdb file', description='')
    parser_a.add_argument("dcd_file", help="Path for dcd file", type=str)
    parser_a.add_argument("pdb_file", help="Path for raw pdb file", type=str)
    parser_a.add_argument("-f", "--file_of_frame_index", help="a list file include wanted frame indexes)", default=None,
                          type=str)
    parser_a.add_argument("-i", "--frame_index", help="frame index split by ,)", default=None,
                          type=str)
    parser_a.add_argument("-o", "--output_prefix", help="Output pdb file name index", default="output_", type=str)
    parser_a.add_argument("-s", "--atom_select",
                          help="atom select string, read http://prody.csb.pitt.edu/manual/reference/atomic/select.html#module-prody.atomic.select",
                          default="not water", type=str)

    # argparse for AtomDist
    parser_a = subparsers.add_parser('AtomDist',
                                     help='get distance between two atoms from dcd file', description='')
    parser_a.add_argument("dcd_file", help="Path for dcd file", type=str)
    parser_a.add_argument("pdb_file", help="Path for raw pdb file", type=str)
    parser_a.add_argument("atom_pair_file", help="two column file)", type=str)
    parser_a.add_argument("-o", "--output_file", help="Output file", default="output.tsv", type=str)

    # argparse for getRMSD
    parser_a = subparsers.add_parser('getRMSD',
                                     help='get RMSD value for a DCD file', description='')
    parser_a.add_argument("dcd_file", help="Path for dcd file", type=str)
    parser_a.add_argument("pdb_file", help="Path for raw pdb file", type=str)
    parser_a.add_argument("-o", "--output_file", help="Output file", default="output.tsv", type=str)
    parser_a.add_argument("-s", "--atom_select",
                          help="atom select string, read http://prody.csb.pitt.edu/manual/reference/atomic/select.html#module-prody.atomic.select",
                          default="not water", type=str)
    # argparse for getRERMSD
    parser_a = subparsers.add_parser('getRERMSD',
                                     help='get RMSD value for a DCD file', description='')
    parser_a.add_argument("dcd_file", help="Path for dcd file", type=str)
    parser_a.add_argument("pdb_file", help="Path for raw pdb file", type=str)
    parser_a.add_argument("rmsd_file", help="Output file", type=str)
    parser_a.add_argument("rmsf_file", help="Output file", type=str)
    parser_a.add_argument("-s", "--atom_select",
                          help="atom select string, read http://prody.csb.pitt.edu/manual/reference/atomic/select.html#module-prody.atomic.select",
                          default="not water", type=str)

    args = parser.parse_args()
    args_dict = vars(args)

    # --------------------------------------------
    #### command detail

    if args_dict["subcommand_name"] == "Dcd2pdb":
        ID_file = args.file_of_frame_index
        ID_list = args.frame_index
        if ID_file:
            ID_list = tsv_file_parse(ID_file, key_col=1)
            ID_list = ID_list.keys()
            ID_list = [int(i) for i in ID_list]
        elif ID_list:
            ID_list = ID_list.split(",")
            ID_list = [int(i) for i in ID_list]
        else:
            ID_list = None

        get_a_frame_from_dcd_to_pdb(args.dcd_file, args.pdb_file, args.output_prefix, index_list=ID_list,
                                    atom_select=args.atom_select)

    elif args_dict["subcommand_name"] == "AtomDist":
        """
        class abc():
            pass
        args = abc()
        
        args.atom_pair_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/atom_pair.txt"
        args.dcd_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/20ns/minimized.dcd"
        args.pdb_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/20ns/solvate.pdb"
        args.output_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/20ns/atom.dis.tsv"
        """

        ID_list = tsv_file_parse(args.atom_pair_file)
        atom_pair_list = [(int(ID_list[i][0]), int(ID_list[i][1])) for i in ID_list]

        output_dir = atom_distance(args.dcd_file, args.pdb_file, atom_pair_list)

        with open(args.output_file, 'w') as f:
            f.write(printer_list(atom_pair_list) + "\n")
            for i in range(0, len(output_dir[atom_pair_list[0]])):
                data_list = [output_dir[j][i] for j in atom_pair_list]
                f.write(printer_list(data_list) + "\n")

    elif args_dict["subcommand_name"] == "getRMSD":
        """
        class abc():
            pass
        args = abc()

        args.dcd_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/n3/20ns/minimized.dcd"
        args.output_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/n3/20ns/RMSD.txt"
        args.pdb_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/n3/20ns/solvate.pdb"
        """

        RMSD_list = get_RMSD(args.dcd_file, args.pdb_file, args.atom_select)

        with open(args.output_file, 'w') as f:
            for i in RMSD_list:
                f.write(str(i) + "\n")

    elif args_dict["subcommand_name"] == "getRERMSD":
        """
        class abc():
            pass
        args = abc()

        args.dcd_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/n3/20ns/minimized.dcd"
        args.rmsd_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/n3/20ns/RMSD.txt"
        args.rmsf_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/n3/20ns/RMSD.txt"
        args.pdb_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190906/n3/20ns/solvate.pdb"
        """

        RMSD_list,RMSF_list = get_RE_RMSD(args.dcd_file, args.pdb_file, args.atom_select)

        with open(args.rmsd_file, 'w') as f:
            for i in RMSD_list:
                f.write(str(i) + "\n")

        with open(args.rmsf_file, 'w') as f:
            for i in RMSF_list:
                f.write(str(i) + "\n")




### solve the longest common subsequence problem

# get the matrix of LCS lengths at each sub-step of the recursive process
# (m+1 by n+1, where m=len(list1) & n=len(list2) ... it's one larger in each direction
# so we don't have to special-case the x-1 cases at the first elements of the iteration
def lcs_mat(list1, list2):
    m = len(list1)
    n = len(list2)
    # construct the matrix, of all zeroes
    mat = [[0] * (n + 1) for row in range(m + 1)]
    # populate the matrix, iteratively
    for row in range(1, m + 1):
        for col in range(1, n + 1):
            if list1[row - 1] == list2[col - 1]:
                # if it's the same element, it's one longer than the LCS of the truncated lists
                mat[row][col] = mat[row - 1][col - 1] + 1
            else:
                # they're not the same, so it's the the maximum of the lengths of the LCSs of the two options (different list truncated in each case)
                mat[row][col] = max(mat[row][col - 1], mat[row - 1][col])
    # the matrix is complete
    return mat


# backtracks all the LCSs through a provided matrix
def all_lcs(lcs_dict, mat, list1, list2, index1, index2):
    # if we've calculated it already, just return that
    if (index1, index2) in lcs_dict: return lcs_dict[(index1, index2)]
    # otherwise, calculate it recursively
    if (index1 == 0) or (index2 == 0):  # base case
        return [[]]
    elif list1[index1 - 1] == list2[index2 - 1]:
        # elements are equal! Add it to all LCSs that pass through these indices
        lcs_dict[(index1, index2)] = [prevs + [list1[index1 - 1]] for prevs in
                                      all_lcs(lcs_dict, mat, list1, list2, index1 - 1, index2 - 1)]
        return lcs_dict[(index1, index2)]
    else:
        lcs_list = []  # set of sets of LCSs from here
        # not the same, so follow longer path recursively
        if mat[index1][index2 - 1] >= mat[index1 - 1][index2]:
            before = all_lcs(lcs_dict, mat, list1, list2, index1, index2 - 1)
            for series in before:  # iterate through all those before
                if not series in lcs_list: lcs_list.append(
                    series)  # and if it's not already been found, append to lcs_list
        if mat[index1 - 1][index2] >= mat[index1][index2 - 1]:
            before = all_lcs(lcs_dict, mat, list1, list2, index1 - 1, index2)
            for series in before:
                if not series in lcs_list: lcs_list.append(series)
        lcs_dict[(index1, index2)] = lcs_list
        return lcs_list


# return a set of the sets of longest common subsequences in list1 and list2
def lcs(list1, list2):
    # mapping of indices to list of LCSs, so we can cut down recursive calls enormously
    mapping = dict()
    # start the process...
    return all_lcs(mapping, lcs_mat(list1, list2), list1, list2, len(list1), len(list2));



from Bio import Phylo
import sys
import re
import os
import matplotlib.pyplot as plt
import matplotlib

matplotlib.use('Agg')

# import lib.base.base_function as bf
# from lib.config import *
# from lib.seq.seq_base import read_fasta_big
# import os
# from lib.file_parser.fileIO import tsv_file_parse
# import lib.evolution.tree_operate_ete3 as to
# import re
# import sys
# from Bio import Phylo
# import networkx as nx
# from lib.seq.blast import evalue_to_wvalue
# import pickle
# from lib.evolution.tree_operate import lookup_by_names, normal_tree_rela_coord, normal_tree_abs_coord, draw_tree, \
#     draw_ascii


# def PtTree(in_file, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num, MSA_flag):
#     if output_dir is None:
#         output_dir = "phyout"

#     output_tag_file = output_dir.split("/")[-1]

#     bf.mkdir(output_dir, keep=True)
#     output_tag = output_dir + "/" + output_tag_file

#     # alignment
#     aln_file = output_tag + ".aa.aln"
#     cmd_string = "sed -i 's/*//g' %s" % (in_file)
#     bf.cmd_run(cmd_string)
#     if MSA_flag == "clustalo":
#         cmd_string = clustalo_path + " -i %s -o %s -t Protein --force --auto --threads=%d" % (
#             in_file, aln_file, cpu_num)
#         bf.cmd_run(cmd_string)
#     elif MSA_flag == "clustalw2":
#         cmd_string = clustalw_path + " -INFILE=%s -ALIGN -OUTPUT=FASTA -OUTFILE=%s -type=protein" % (
#             in_file, aln_file)
#         bf.cmd_run(cmd_string)

#     # trimAl
#     if trimal_flag:
#         used_aln = output_tag + ".trimal.aln"
#         used_aln_for_raxml = output_tag_file + ".trimal.aln"
#         cmd_string = trimal_path + " -fasta -gappyout -in %s -out %s" % (aln_file, used_aln)
#         bf.cmd_run(cmd_string)
#     else:
#         used_aln = aln_file
#         used_aln_for_raxml = output_tag_file + ".aa.aln"

#     # tree builder
#     tree_file = output_tag + ".phb"
#     tree_file_for_raxml = output_tag_file + ".phb"
#     if phylogeneic_tree_builder == 'fasttree':
#         cmd_string = fasttree_path + " -wag -gamma -out %s %s" % (tree_file, used_aln)
#         bf.cmd_run(cmd_string)
#     elif phylogeneic_tree_builder == 'raxml':
#         cmd_string = raxml_path + " -T %d -f a -x 12345 -p 12345 -# 100 -m PROTGAMMAJTT -s %s -n %s" % (
#             cpu_num, used_aln_for_raxml, tree_file_for_raxml)
#         cwd_string = output_dir
#         bf.cmd_run(cmd_string, cwd_string)

#     print("Finished")


# def TrueNtTree(in_file, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num, MSA_flag):
#     if output_dir is None:
#         output_dir = "phyout"

#     bf.mkdir(output_dir)
#     output_tag = output_dir + "/" + output_dir

#     # alignment
#     aln_file = output_tag + ".nt.aln"

#     if MSA_flag == "clustalo":
#         cmd_string = clustalo_path + " -i %s -o %s --force --auto --threads=%d" % (in_file, aln_file, cpu_num)
#         bf.cmd_run(cmd_string)
#     elif MSA_flag == "clustalw2":
#         cmd_string = clustalw_path + " -INFILE=%s -ALIGN -OUTPUT=FASTA -OUTFILE=%s -type=DNA" % (
#             in_file, aln_file)
#         bf.cmd_run(cmd_string)

#     # trimAl
#     if trimal_flag:
#         used_aln = output_tag + ".trimal.aln"
#         used_aln_for_raxml = output_dir + ".trimal.aln"
#         cmd_string = trimal_path + " -fasta -gappyout -in %s -out %s" % (aln_file, used_aln)
#         bf.cmd_run(cmd_string)
#     else:
#         used_aln = aln_file
#         used_aln_for_raxml = output_dir + ".nt.aln"

#     # tree builder
#     tree_file = output_tag + ".phb"
#     tree_file_for_raxml = output_dir + ".phb"
#     if phylogeneic_tree_builder == 'fasttree':
#         cmd_string = fasttree_path + " -nt -gtr -gamma -out %s %s" % (tree_file, used_aln)
#         bf.cmd_run(cmd_string)
#     elif phylogeneic_tree_builder == 'raxml':
#         cmd_string = raxml_path + " -T %d -f a -x 12345 -p 12345 -# 100 -m GTRGAMMA -s %s -n %s" % (
#             cpu_num, used_aln_for_raxml, tree_file_for_raxml)
#         cwd_string = os.getcwd() + "/" + output_dir
#         bf.cmd_run(cmd_string, cwd_string)

#     print("Finished")


# def PtNtTree(nucl_file, pt_file, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num):
#     if output_dir is None:
#         output_dir = "phyout"

#     bf.mkdir(output_dir)
#     output_tag = output_dir + "/" + output_dir

#     # alignment
#     pt_aln_file = output_tag + ".aa.aln"
#     cmd_string = "sed -i 's/*//g' %s" % (pt_file)
#     bf.cmd_run(cmd_string)
#     cmd_string = clustalo_path + " -i %s -o %s -t Protein --force --auto --threads=%d" % (
#         pt_file, pt_aln_file, cpu_num)
#     bf.cmd_run(cmd_string)

#     # backtrans
#     pt_aln_file = output_tag + ".aa.aln"
#     nt_aln_file = output_tag + ".nt.aln"
#     cmd_string = treebest_path + " backtrans %s %s > %s" % (pt_aln_file, nucl_file, nt_aln_file)
#     bf.cmd_run(cmd_string)

#     # trimAl
#     if trimal_flag:
#         used_aln = output_tag + ".trimal.aln"
#         used_aln_for_raxml = output_dir + ".trimal.aln"
#         cmd_string = trimal_path + " -fasta -gappyout -in %s -out %s" % (nt_aln_file, used_aln)
#         bf.cmd_run(cmd_string)
#     else:
#         used_aln = nt_aln_file
#         used_aln_for_raxml = output_dir + ".nt.aln"

#     # tree builder
#     tree_file = output_tag + ".phb"
#     tree_file_for_raxml = output_dir + ".phb"
#     if phylogeneic_tree_builder == 'fasttree':
#         cmd_string = fasttree_path + " -nt -gtr -gamma -out %s %s" % (tree_file, used_aln)
#         bf.cmd_run(cmd_string)
#     elif phylogeneic_tree_builder == 'raxml':
#         cmd_string = raxml_path + " -T %d -f a -x 12345 -p 12345 -# 100 -m GTRGAMMA -s %s -n %s" % (
#             cpu_num, used_aln_for_raxml, tree_file_for_raxml)
#         cwd_string = os.getcwd() + "/" + output_dir
#         bf.cmd_run(cmd_string, cwd_string)

#     print("Finished")


# def FullFamily(query, database, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num):
#     # ToDO
#     pass


# def PartFamily(query, database, num_top, phylogeneic_tree_builder, trimal_flag, mcl_flag, makeblastdb_flag, output_tag,
#                cpu_num):
#     from functools import cmp_to_key
#     if output_tag is None:
#         output_tag = query
#     output_tag = output_tag.split("/")[-1]
#     output_dir = os.getcwd() + "/" + output_tag
#     bf.mkdir(output_dir, keep=True)
#     output_tag = output_dir + "/" + output_tag

#     ##

#     if makeblastdb_flag is True:
#         for files in os.listdir(database):
#             if not files.split(".")[-1] == "fasta":
#                 continue
#             cmd_string = "makeblastdb -in %s -dbtype prot" % (database + "/" + files)
#             bf.cmd_run(cmd_string)

#     query_record = ""
#     for files in os.listdir(database):
#         if files.split(".")[-1] in ("phr", "pin", "psq"):
#             continue
#         for record in read_fasta_big(database + "/" + files):
#             if record.seqname_short() == query:
#                 query_record = record
#                 break
#         if query_record != "":
#             break
#     query_fa = output_tag + ".query.fa"
#     with open(query_fa, "w") as f:
#         f.write(">" + query_record.seqname_short() + "\n" + query_record.seqs)

#     # blast
#     blast_result = []
#     for files in os.listdir(database):
#         if files.split(".")[-1] in ("phr", "pin", "psq"):
#             continue
#         bls_out = output_tag + "_vs_" + files
#         db_file = database + "/" + files
#         cmd_string = "blastp -query %s -db %s -out %s -evalue 1e-5 -outfmt 6 -num_threads %d -max_target_seqs %d" % (
#             query_fa, db_file, bls_out, cpu_num, num_top)
#         bf.cmd_run(cmd_string)
#         blast_result.append(bls_out)

#     subject_id = []
#     for bls_file in blast_result:
#         subject_id.extend(list(tsv_file_parse(bls_file, key_col=2).keys()))
#     subject_record = []
#     for files in os.listdir(database):
#         if files.split(".")[-1] in ("phr", "pin", "psq"):
#             continue
#         for record in read_fasta_big(database + "/" + files):
#             if record.seqname_short() in subject_id:
#                 subject_record.append(record)
#     subject_fa = output_tag + ".blast.subject.fa"
#     with open(subject_fa, "w") as f:
#         for record in subject_record:
#             f.write(">" + record.seqname_short() + "\n" + record.seqs + "\n")

#     # mcl cluster
#     if mcl_flag is True:
#         # all to all blast
#         cmd_string = "makeblastdb -in %s -dbtype prot" % subject_fa
#         bf.cmd_run(cmd_string)
#         all_to_all_blast = output_tag + ".subject.all_to_all.bls"
#         cmd_string = "blastp -query %s -db %s -out %s -evalue 1e-5 -outfmt 6 -num_threads %d" % (
#             subject_fa, subject_fa, all_to_all_blast, cpu_num)
#         bf.cmd_run(cmd_string)

#         # mcl pre
#         tsv_dict = tsv_file_parse(all_to_all_blast, fields="1,2,11")
#         mcl_abc_file = output_tag + ".seq.abc"
#         with open(mcl_abc_file, "w") as f:
#             for a, b, c in [tsv_dict[i] for i in tsv_dict]:
#                 f.write(a + "\t" + b + "\t" + c + "\n")

#         # mcl
#         mcl_mci_file = output_tag + ".seq.mci"
#         mcl_tab_file = output_tag + ".seq.tab"
#         cmd_string = "mcxload -abc %s --stream-mirror --stream-neg-log10 -stream-tf 'ceil(200)' -o %s -write-tab %s" % (
#             mcl_abc_file, mcl_mci_file, mcl_tab_file)
#         bf.cmd_run(cmd_string)

#         inflation = 1.5
#         mcl_out_files = []
#         mcl_out_group = {}
#         while (inflation <= 6.0):
#             mcl_output = "%s.%.1f.mcl.out" % (output_tag, inflation)
#             cmd_string = "mcl %s -I %s -use-tab %s -o %s" % (mcl_mci_file, inflation, mcl_tab_file, mcl_output)
#             bf.cmd_run(cmd_string)
#             mcl_out_files.append(mcl_output)
#             mcl_output_dir = tsv_file_parse(mcl_output)
#             for group_id in mcl_output_dir:
#                 if query in mcl_output_dir[group_id]:
#                     mcl_out_group[inflation] = mcl_output_dir[group_id]
#             inflation = inflation + 0.5

#         def cmp_tmp(x, y):
#             x_speci_list = list(set([i.split("|")[0] for i in mcl_out_group[x]]))
#             y_speci_list = list(set([i.split("|")[0] for i in mcl_out_group[y]]))

#             if len(x_speci_list) > len(y_speci_list):
#                 return -1
#             elif len(x_speci_list) < len(y_speci_list):
#                 return 1
#             elif len(x_speci_list) == len(y_speci_list):
#                 if x > y:
#                     return -1
#                 elif y > x:
#                     return 1

#         best_mcl = sorted(mcl_out_group, key=cmp_to_key(cmp_tmp))[0]
#         best_mcl_list = mcl_out_group[best_mcl]

#         mcl_cluster_fa = output_tag + ".mcl.fa"
#         with open(mcl_cluster_fa, "w") as f:
#             for record in subject_record:
#                 if record.seqname_short() in best_mcl_list:
#                     f.write(">" + record.seqname_short() + "\n" + record.seqs + "\n")

#         tree_fa = mcl_cluster_fa
#     else:
#         tree_fa = subject_fa

#     PtTree(tree_fa, phylogeneic_tree_builder, trimal_flag, output_dir, cpu_num)


if __name__ == '__main__':

    import argparse

    # argument parse
    parser = argparse.ArgumentParser(
        prog='PhylTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for PtTree
    parser_a = subparsers.add_parser('PtTree',
                                     help='build a protein gene tree from raw sequence',
                                     description='build a protein gene tree from raw sequence')

    parser_a.add_argument('protein_fasta', type=str,
                          help='Path of raw protein sequences')
    parser_a.add_argument("-p", "--phylogeneic_tree_builder", help="what tree builder you want? (default:fasttree)",
                          default="fasttree", choices=['raxml', 'fasttree'])
    parser_a.add_argument("-c", "--MSA", help="what multiple sequence alignment you want? (default:clustalo)",
                          default="clustalo", choices=['clustalo', 'clustalw2'])
    parser_a.add_argument(
        "-t", "--trimal", help="use trimal to trim alignment", action='store_true')
    parser_a.add_argument(
        "-T", "--num_threads", help="num of threads (default:1)", default=1, type=int)
    parser_a.add_argument("-o", "--output_dir", help="the dir name for all output file (default:phyout)", default=None,
                          type=str)

    # argparse for TrueNtTree
    parser_a = subparsers.add_parser('TrueNtTree',
                                     help='build a nucl gene tree from raw sequence',
                                     description='just need nucl information')

    parser_a.add_argument('nucl_fasta', type=str,
                          help='Path of raw nucl sequences')
    parser_a.add_argument("-p", "--phylogeneic_tree_builder", help="what tree builder you want? (default:fasttree)",
                          default="fasttree", choices=['raxml', 'fasttree'])
    parser_a.add_argument("-c", "--MSA", help="what multiple sequence alignment you want? (default:clustalo)",
                          default="clustalo", choices=['clustalo', 'clustalw2'])
    parser_a.add_argument(
        "-t", "--trimal", help="use trimal to trim alignment", action='store_true')
    parser_a.add_argument(
        "-T", "--num_threads", help="num of threads (default:1)", default=1, type=int)
    parser_a.add_argument("-o", "--output_dir", help="the dir name for all output file (default:phyout)", default=None,
                          type=str)

    # argparse for PtNtTree
    parser_a = subparsers.add_parser('PtNtTree',
                                     help='build a coden gene tree from nt aligned sequence based on protein alignment',
                                     description='need nucl and protein information')

    parser_a.add_argument('nucl_fasta', type=str,
                          help='Path of raw nucl sequences')
    parser_a.add_argument('protein_fasta', type=str,
                          help='Path of raw protein sequences')
    parser_a.add_argument("-p", "--phylogeneic_tree_builder", help="what tree builder you want? (default:fasttree)",
                          default="fasttree", choices=['raxml', 'fasttree'])
    parser_a.add_argument(
        "-t", "--trimal", help="use trimal to trim alignment", action='store_true')
    parser_a.add_argument(
        "-T", "--num_threads", help="num of threads (default:1)", default=1, type=int)
    parser_a.add_argument("-o", "--output_dir", help="the dir name for all output file (default:phyout)", default=None,
                          type=str)

    # argparse for Print
    parser_a = subparsers.add_parser('Print',
                                     help='print a tree by Ascii on screen',
                                     description='print a tree by Ascii on screen')

    parser_a.add_argument('tree_file', type=str,
                          help='tree file with newick format')
    parser_a.add_argument(
        "-o", "--output_file", help="output file (default:stdout)", default=None, type=str)
    parser_a.add_argument(
        "-t", "--target_list", help="leaf in target list will be highlighted", default="", type=str)
    parser_a.add_argument("-f", "--file_format", help="tree file format (default:newick)",
                          default="newick", choices=['newick', 'phyloxml'])

    # argparse for SubTree
    parser_a = subparsers.add_parser('SubTree',
                                     help='cut a sub tree from a big tree by MRCA of two leaves',
                                     description='cut a sub tree from a big tree')

    parser_a.add_argument('tree_file', type=str,
                          help='tree file with newick format')
    parser_a.add_argument('leaf1', type=str, help='leaf 1')
    parser_a.add_argument('leaf2', type=str, help='leaf 2')
    parser_a.add_argument(
        "-r", "--root", help="re-root tree by this leaf", type=str)
    parser_a.add_argument("-o", "--output_file", help="output file", type=str)
    parser_a.add_argument(
        "-n", "--name_flag", help="output the name list of subtree", action='store_true')

    # argparse for Newick2PhyloXML
    parser_a = subparsers.add_parser('Newick2PhyloXML',
                                     help='change Newick file to PhyloXML')

    parser_a.add_argument('input_tree', type=str, help='path for input tree')
    parser_a.add_argument('output_tree', type=str, help='path for output tree')

    # argparse for RenameNodeID
    parser_a = subparsers.add_parser('RenameNodeID',
                                     help='Rename node on a tree')

    parser_a.add_argument('input_tree', type=str, help='path for input tree')
    parser_a.add_argument('output_tree', type=str, help='path for output tree')
    parser_a.add_argument('ID_map_file', type=str,
                          help='file with ID in first column and new ID in a second column')
    parser_a.add_argument("-f", "--file_format", help="tree file format (default:phyloxml)",
                          default="newick", choices=['newick', 'phyloxml'])

    # argparse for AddNodeID
    parser_a = subparsers.add_parser('AddNodeID',
                                     help='Add ordered ID for node in the tree')

    parser_a.add_argument('input_tree', type=str, help='path for input tree')
    parser_a.add_argument('output_tree', type=str, help='path for output tree')
    parser_a.add_argument("-f", "--file_format", help="tree file format (default:phyloxml)",
                          default="phyloxml", choices=['newick', 'phyloxml'])

    # argparse for ClearNodeID
    parser_a = subparsers.add_parser('ClearNodeID',
                                     help='clear ID for nodes in the tree')

    parser_a.add_argument('input_tree', type=str, help='path for input tree')
    parser_a.add_argument('output_tree', type=str, help='path for output tree')
    parser_a.add_argument("-f", "--file_format", help="tree file format (default:phyloxml)",
                          default="phyloxml", choices=['newick', 'phyloxml'])

    # argparse for ClearConfidence
    parser_a = subparsers.add_parser('ClearConfidence',
                                     help='clear confidence for nodes in the tree')

    parser_a.add_argument('input_tree', type=str, help='path for input tree')
    parser_a.add_argument('output_tree', type=str, help='path for output tree')
    parser_a.add_argument("-f", "--file_format", help="tree file format (default:phyloxml)",
                          default="phyloxml", choices=['newick', 'phyloxml'])

    # argparse for ClearComment
    parser_a = subparsers.add_parser('ClearComment',
                                     help='clear comment for nodes in the tree')

    parser_a.add_argument('input_tree', type=str, help='path for input tree')
    parser_a.add_argument('output_tree', type=str, help='path for output tree')
    parser_a.add_argument("-f", "--file_format", help="tree file format (default:phyloxml)",
                          default="phyloxml", choices=['newick', 'phyloxml'])

    # argparse for RootTree
    parser_a = subparsers.add_parser('RootTree',
                                     help='Root a Tree with outgroup or at midpoint')

    parser_a.add_argument('input_tree', type=str, help='path for input tree')
    parser_a.add_argument('output_tree', type=str, help='path for output tree')
    parser_a.add_argument('-n', '--node_name', type=str,
                          help='node name for reroot, default is root at midpoint')
    parser_a.add_argument("-f", "--file_format", help="tree file format (default:phyloxml)",
                          default="phyloxml", choices=['newick', 'phyloxml'])

    # argparse for RootTreeBySpecies
    parser_a = subparsers.add_parser('RootTreeBySpecies',
                                     help='Root a Gene Tree by species tree')

    parser_a.add_argument(
        "gene_tree", help="input gene tree in newick", type=str)
    parser_a.add_argument(
        "species_tree", help="species tree in newick", type=str)
    parser_a.add_argument(
        "gene_to_species_map", help="gene name to species map file in tsv file, gene id in first col, species id in second", type=str)
    parser_a.add_argument("-o", "--output_tree",
                          help="output tree name", type=str, default="gene_rooted.phb")

    # argparse for MonophylyTest
    parser_a = subparsers.add_parser('MonophylyTest',
                                     help='use Bootstrap tree to support a list leaves is monophyly or not')

    parser_a.add_argument('gene_list_file', type=str,
                          help='path for gene list file')
    parser_a.add_argument('Bootstrap_trees', type=str,
                          help='path for file which have many bootstrap trees')
    parser_a.add_argument("-f", "--file_format", help="tree file format (default:newick)",
                          default="newick", choices=['newick', 'phyloxml'])
    parser_a.add_argument(
        "-o", "--output_file", help="output file (default:stdout)", default=None, type=str)

    # argparse for FullFamily
    parser_a = subparsers.add_parser('FullFamily',
                                     help='build a full family gene tree by hmm searching for a database',
                                     description='build a full family gene tree by hmm search a database')

    parser_a.add_argument(
        'query', type=str, help='query sequences file in fasta format')
    parser_a.add_argument('database', type=str,
                          help='all sequences file in fasta format')
    parser_a.add_argument(
        "-t", "--trimal", help="use trimal to trim alignment", action='store_true')
    parser_a.add_argument(
        "-T", "--num_threads", help="num of threads (default:1)", default=1, type=int)
    parser_a.add_argument("-p", "--phylogeneic_tree_builder", help="what tree builder you want? (default:fasttree)",
                          default="fasttree", choices=['raxml', 'fasttree'])
    parser_a.add_argument("-o", "--output_prefix",
                          help="prefix for all output", default="phyout", type=str)

    # argparse for PartFamily
    parser_a = subparsers.add_parser('PartFamily',
                                     help='build a Part family gene tree by top blast searching for a every species',
                                     description='build a Part family gene tree by top blast searching for a every species')

    parser_a.add_argument('query', type=str, help='query id')
    parser_a.add_argument('database', type=str,
                          help='a dir in which each species sequence is placed in a separate file')
    parser_a.add_argument("-n", "--num_top", help="num of top gene from each species (default:20)", default=20,
                          type=int)
    parser_a.add_argument(
        "-t", "--trimal", help="use trimal to trim alignment", action='store_true')
    parser_a.add_argument(
        "-m", "--mcl", help="use mcl to cluster subject from blast", action='store_true')
    parser_a.add_argument("-b", "--blast_db_maker", help="makeblastdb for fasta file, only need in first run",
                          action='store_true')
    parser_a.add_argument(
        "-T", "--num_threads", help="num of threads (default:1)", default=1, type=int)
    parser_a.add_argument("-p", "--phylogeneic_tree_builder", help="what tree builder you want? (default:fasttree)",
                          default="fasttree", choices=['raxml', 'fasttree'])
    parser_a.add_argument("-o", "--output_prefix",
                          help="prefix for all output", default="phyout", type=str)

    # argparse for OrthofinderParser
    parser_a = subparsers.add_parser('OrthofinderParser',
                                     help='parser orthofinder recon tree for orthology')

    parser_a.add_argument('recon_tree', type=str,
                          help='path for recon tree build by orthofinder')
    parser_a.add_argument('speci_tree', type=str,
                          help='path for species tree build by orthofinder (SpeciesTree_rooted_node_labels.txt)')
    parser_a.add_argument('base_speci_tab', type=str,
                          help='tab of species which you want to be base species for orthology find')
    parser_a.add_argument("-o", "--output_prefix",
                          help="prefix for all output", default="phyout", type=str)

    # argparse for mcl
    parser_a = subparsers.add_parser('mcl',
                                     help='make mcl for a fasta file')

    parser_a.add_argument('input_fasta', type=str,
                          help='path for input fasta file')
    parser_a.add_argument('output_prefix', type=str,
                          help='prefix for output')
    parser_a.add_argument("-t", "--cpu_num", type=int,
                          help='cpu num', default=5)
    parser_a.add_argument("-bls", "--blast_output",
                          help="give a output of all 2 all blast for skip blast", type=str)

    # argparse for WvalueGroup
    parser_a = subparsers.add_parser('WvalueGroup',
                                     help='')

    parser_a.add_argument('all_2_all_blast_results', type=str,
                          help='path for input blast results outfmt 6')
    parser_a.add_argument('node_group_output_file',
                          type=str, help='prefix for output')
    parser_a.add_argument("-t", "--threshold", type=int,
                          help='group threshold', default=100)

    # argparse for blastEtoW
    parser_a = subparsers.add_parser('blastEtoW',
                                     help='')

    parser_a.add_argument('a2a_blast', type=str,
                          help='path for input blast results outfmt 6')
    parser_a.add_argument('w_value_file', type=str, help='output file')

    # argparse for EasyABSREL
    parser_a = subparsers.add_parser('EasyABSREL',
                                     help='run ABSREL quickly')

    parser_a.add_argument('cds_aln_fasta', type=str,
                          help='path for input cds aln fasta')
    parser_a.add_argument('rooted_gene_tree', type=str,
                          help='path for rooted gene tree')
    parser_a.add_argument('-o', '--output_prefix', type=str,
                          help='path for output', default='ABSREL')
    parser_a.add_argument('-t', '--threads', type=int,
                          help='CPU for ABSREL', default=8)
    parser_a.add_argument('-f', '--tree_format', type=str,
                          help='tree format', default='newick')

    # argparse for EasyRELAX
    parser_a = subparsers.add_parser('EasyRELAX',
                                     help='run RELAX quickly')

    parser_a.add_argument('cds_aln_fasta', type=str,
                          help='path for input cds aln fasta')
    parser_a.add_argument('rooted_gene_tree', type=str,
                          help='path for rooted gene tree')
    parser_a.add_argument('output_dir', type=str,
                          help='path for output_dir')
    parser_a.add_argument('-n', '--test_node', type=str, default="",
                          help='test node list "N1,N2,N3"')
    parser_a.add_argument('-f', '--tree_format', type=str, default="newick",
                          help='tree file format')
    parser_a.add_argument('-t', '--threads', type=int, default=8,
                          help='num of threads')

    # argparse for parserSSN
    parser_a = subparsers.add_parser('parserSSN',
                                     help='parse blast results to Sequence similarity network')

    parser_a.add_argument('a2a_blast', type=str,
                          help='path for input blast results outfmt 6')
    parser_a.add_argument('graph_pyb_file', type=str, help='graph file output')
    parser_a.add_argument("graph_pos_file", type=str, help='pos file output')

    # argparse for SSNplot
    parser_a = subparsers.add_parser('SSNplot',
                                     help='parse blast results to Sequence similarity network')

    parser_a.add_argument('graph_pyb_file', type=str, help='graph file')
    parser_a.add_argument('graph_pos_file', type=str, help='pos file')
    parser_a.add_argument("plot_file", type=str, help='plot pdf')

    # argparse for BlastTo1KP
    parser_a = subparsers.add_parser('BlastTo1KP',
                                     help='parse blast results from a genome to 1kp')

    parser_a.add_argument('blast_result', type=str,
                          help='path for blast_result')
    parser_a.add_argument('taxon_dir', type=str, help='path for taxon_dir')
    parser_a.add_argument("acc_taxon_map", type=str,
                          help='path for acc_taxon_map')

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    if args_dict["subcommand_name"] == "PtTree":
        from src.xuyuxing.tools.phytools import PtTree

        in_file = args.protein_fasta
        phylogeneic_tree_builder = args.phylogeneic_tree_builder
        trimal_flag = args.trimal
        output_dir = args.output_dir
        cpu_num = args.num_threads
        MSA = args.MSA

        PtTree(in_file, phylogeneic_tree_builder,
               trimal_flag, output_dir, cpu_num, MSA)

    elif args_dict["subcommand_name"] == "TrueNtTree":
        from src.xuyuxing.tools.phytools import TrueNtTree

        in_file = args.nucl_fasta
        phylogeneic_tree_builder = args.phylogeneic_tree_builder
        trimal_flag = args.trimal
        output_dir = args.output_dir
        cpu_num = args.num_threads
        MSA = args.MSA

        TrueNtTree(in_file, phylogeneic_tree_builder,
                   trimal_flag, output_dir, cpu_num, MSA)

    elif args_dict["subcommand_name"] == "PtNtTree":
        from src.xuyuxing.tools.phytools import PtNtTree

        nucl_file = args.nucl_fasta
        pt_file = args.protein_fasta
        phylogeneic_tree_builder = args.phylogeneic_tree_builder
        trimal_flag = args.trimal
        output_dir = args.output_dir
        cpu_num = args.num_threads

        PtNtTree(nucl_file, pt_file, phylogeneic_tree_builder,
                 trimal_flag, output_dir, cpu_num)

    elif args_dict["subcommand_name"] == "Print":
        from lib.common.evolution.tree_operate import draw_ascii

        tree = Phylo.read(args.tree_file, args.file_format)

        draw_ascii(tree, args.output_file, clade_name=True)

        # tree_file = args.tree_file
        # target_list = args.target_list
        # target_list = target_list.split(",")
        #
        # t = Tree(tree_file)
        # # tree_string = t.get_ascii(show_internal=True, attributes=["name", "dist", "label", "support"])
        # tree_string = t.get_ascii(show_internal=True, attributes=["name", "support"])
        #
        # target_list = list(set(target_list))
        # for target in target_list:
        #     tree_string = tree_string.replace(target, "\033[0;31m" + target + "\033[0m")
        #
        # print(tree_string)

    elif args_dict["subcommand_name"] == "SubTree":
        import lib.xuyuxing.evolution.tree_operate_ete3 as to

        """
            tree_file = '/lustre/home/xuyuxing/Work/Cuscuta_HGT/123/HGT_case/case_001/genblastG/mcl/Cauv1.1_5.blast/Cauv1.1_5.blast.taxon.phb'
            leaf1 = '4039_0'
            leaf2 = '35608_14'
            root_node = None
            """

        tree_file = args.tree_file
        leaf1 = args.leaf1
        leaf2 = args.leaf2
        root_node = args.root
        output_file = args.output_file
        name_flag = args.name_flag

        subnode = to.subtree(tree_file, leaf1, leaf2, root_node)
        if name_flag:
            if output_file is not None:
                F1 = open(output_file, 'w')

            for i in subnode.get_leaf_names():
                if output_file is not None:
                    F1.write(i + "\n")
                else:
                    print(i)

            if output_file is not None:
                F1.close()

        else:
            if output_file is not None:
                with open(output_file, 'w') as f:
                    f.write(subnode.write())
            else:
                print(subnode)

    elif args_dict["subcommand_name"] == "FullFamily":
        # ToDo
        pass

    elif args_dict["subcommand_name"] == "PartFamily":
        from src.xuyuxing.tools.phytools import PartFamily

        query = args.query
        database = args.database
        num_top = args.num_top
        phylogeneic_tree_builder = args.phylogeneic_tree_builder
        trimal_flag = args.trimal
        mcl_flag = args.mcl
        makeblastdb_flag = args.blast_db_maker
        output_tag = args.output_prefix
        cpu_num = args.num_threads

        PartFamily(query, database, num_top, phylogeneic_tree_builder, trimal_flag, mcl_flag, makeblastdb_flag,
                   output_tag, cpu_num)

    elif args_dict["subcommand_name"] == "OrthofinderParser":
        """
        class abc(object):
            pass

        args = abc()

        args.recon_tree = '/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10/Orthologues_Apr11/Recon_Gene_Trees/OG0000001_tree.txt'
        args.speci_tree = '/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10/Orthologues_Apr11/SpeciesTree_rooted_node_labels.txt'
        args.base_speci_tab = 'Ini'
        args.output_file = '/lustre/home/xuyuxing/Work/Csp/orthofinder/Orthologues/tmp.out'
        """
        from ete3 import Tree

        def read_orthofinder_lab_tree(tree_file):
            with open(tree_file, "r") as myfile:
                data = myfile.read()
                tree_string = re.sub('\)n0;$', ');', data)
                tree_string = re.sub('\)N0;$', ');', tree_string)
                t = Tree(tree_string, format=3)
            return t

        t_r = read_orthofinder_lab_tree(args.recon_tree)
        t_s = read_orthofinder_lab_tree(args.speci_tree)

    elif args_dict["subcommand_name"] == "mcl":
        """
        class abc(object):
            pass

        args = abc()

        args.input_fasta = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/mcl/sj90'
        args.output_prefix = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/mcl/sj90_mcl'
        args.cpu_num=5
        args.blast_output = None
        """
        from lib.common.os import cmd_run

        work_dir = os.path.dirname(args.input_fasta)

        # mcl cluster
        # all to all blast
        if args.blast_output is None:
            cmd_string = "makeblastdb -in %s -dbtype prot" % args.input_fasta
            cmd_run(cmd_string)
            all_to_all_blast = args.input_fasta + ".all_to_all.bls"
            cmd_string = "blastp -query %s -db %s -out %s -evalue 1e-5 -outfmt 6 -num_threads %d" % (
                args.input_fasta, args.input_fasta, all_to_all_blast, args.cpu_num)
            cmd_run(cmd_string)
        else:
            all_to_all_blast = args.blast_output

        # mcl pre
        mcl_abc_file = args.output_prefix + ".seq.abc"
        with open(all_to_all_blast, 'r') as r:
            with open(mcl_abc_file, "w") as f:
                for each_line in r:
                    if re.match(r'^#', each_line):
                        continue
                    each_line = re.sub('\n', '', each_line)
                    info = each_line.split("\t")
                    a, b, c = info[0], info[1], info[10]
                    f.write(a + "\t" + b + "\t" + c + "\n")

        # mcl
        mcl_mci_file = args.output_prefix + ".seq.mci"
        mcl_tab_file = args.output_prefix + ".seq.tab"
        cmd_string = "mcxload -abc %s --stream-mirror --stream-neg-log10 -stream-tf 'ceil(200)' -o %s -write-tab %s" % (
            mcl_abc_file, mcl_mci_file, mcl_tab_file)
        cmd_run(cmd_string)

        inflation = 1.5
        mcl_out_files = []
        mcl_out_group = {}
        while (inflation <= 6.0):
            mcl_output = "%s.%.1f.mcl.out" % (args.output_prefix, inflation)
            cmd_string = "mcl %s -I %s -use-tab %s -o %s" % (
                mcl_mci_file, inflation, mcl_tab_file, mcl_output)
            cmd_run(cmd_string)
            inflation = inflation + 0.5

    elif args_dict["subcommand_name"] == "WvalueGroup":
        """
        class abc(object):
            pass

        args = abc()

        args.all_2_all_blast_results = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/mcl/tmp/sj90_a2a.bls'
        args.node_group_output_file = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/mcl/tmp/sj90_a2a.node_group.tsv' 
        args.threshold = 80
        """

        # group seq to node (by a thre)
        import networkx as nx
        from api.common.genome.blast import evalue_to_wvalue

        G = nx.Graph()

        with open(args.all_2_all_blast_results, 'r') as r:
            num = 0
            for each_line in r:
                num = num + 1
                if re.match(r'^#', each_line):
                    continue
                each_line = re.sub('\n', '', each_line)
                info = each_line.split("\t")
                seq1, seq2, evalue = info[0], info[1], info[10]
                wvalue = evalue_to_wvalue(float(evalue))
                G.add_nodes_from([seq1, seq2])
                if wvalue >= args.threshold:
                    G.add_edge(seq1, seq2)
                if num % 1000000 == 0:
                    print("added %s records to graph" % num)
                    tmp_G = nx.Graph()
                    tmp_G.add_nodes_from(list(G.node))
                    for sub_graph in (G.subgraph(c) for c in nx.connected_components(G)):
                        nodes_in_sub_graph = list(sub_graph.node)
                        tmp_G.add_edges_from([(nodes_in_sub_graph[i], nodes_in_sub_graph[i + 1]) for i in
                                              range(0, len(nodes_in_sub_graph) - 1)])
                    G = tmp_G

        num = 0
        with open(args.node_group_output_file, 'w') as f:
            for sub_graph in (G.subgraph(c) for c in nx.connected_components(G)):
                num = num + 1
                for node in sub_graph.node:
                    f.write("%d\t%s\n" % (num, node))

    elif args_dict["subcommand_name"] == "blastEtoW":
        """
        class abc(object):
            pass

        args = abc()

        args.a2a_blast = '/Users/yuxingxu/Job/Work/FKN/all.tmp.bls'
        args.w_value_file = '/Users/yuxingxu/Job/Work/FKN/graph.py'
        """

        import matplotlib.pyplot as plt
        import networkx as nx
        import re
        import pickle

        G = nx.Graph()

        with open(args.a2a_blast, 'r') as r:
            num = 0
            for each_line in r:
                num = num + 1
                if re.match(r'^#', each_line):
                    continue
                each_line = re.sub('\n', '', each_line)
                info = each_line.split("\t")
                seq1, seq2, evalue = info[0], info[1], info[10]
                wvalue = evalue_to_wvalue(float(evalue))
                G.add_nodes_from([seq1, seq2])
                G.add_weighted_edges_from([(seq1, seq2, wvalue)])

        with open(args.w_value_file, 'w') as w:
            for edge_node_pair in G.edges:
                wvalue = G.edges[edge_node_pair]['weight']
                w.write("%s\t%s\t%d\n" %
                        (edge_node_pair[0], edge_node_pair[1], wvalue))

    elif args_dict["subcommand_name"] == "parserSSN":
        """
        class abc(object):
            pass

        args = abc()

        args.a2a_blast = '/Users/yuxingxu/Job/Work/FKN/all.tmp.bls'
        args.graph_pyb_file = '/Users/yuxingxu/Job/Work/FKN/graph.py'
        args.graph_pos_file ='/Users/yuxingxu/Job/Work/FKN/graph.pos.py' 
        """

        import matplotlib.pyplot as plt
        import networkx as nx
        import re
        import pickle

        G = nx.Graph()

        with open(args.a2a_blast, 'r') as r:
            num = 0
            for each_line in r:
                num = num + 1
                if re.match(r'^#', each_line):
                    continue
                each_line = re.sub('\n', '', each_line)
                info = each_line.split("\t")
                seq1, seq2, evalue = info[0], info[1], info[10]
                wvalue = evalue_to_wvalue(float(evalue))
                G.add_nodes_from([seq1, seq2])
                G.add_weighted_edges_from([(seq1, seq2, wvalue)])
                if num % 1000000 == 0:
                    print("added %s records to graph" % num)

        pos = nx.spring_layout(G)

        nx.write_gpickle(G, args.graph_pyb_file)
        with open(args.graph_pos_file, 'wb') as f:
            pickle.dump(pos, f)

    elif args_dict["subcommand_name"] == 'SSNplot':
        """
        class abc(object):
            pass

        args = abc()


        args.graph_pyb_file = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/sj50/graph/tmp/graph.py'
        args.graph_pos_file ='/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/sj50/graph/tmp/graph.pos.py' 
        args.plot_file = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/sj50/graph/tmp/graph.pdf'
        """
        import networkx as nx

        with open(args.graph_pos_file, 'rb') as f:
            pos = pickle.load(f)

        G = nx.read_gpickle(args.graph_pyb_file)

        plt.figure(figsize=(8, 8))
        nx.draw_networkx_nodes(G, pos,
                               node_size=10)
        # nx.draw_networkx_labels(G, pos)
        nx.draw_networkx_edges(G, pos, alpha=0.4)
        plt.axis('off')
        # plt.show()
        plt.savefig(args.plot_file, dpi=1000)

    elif args_dict["subcommand_name"] == "GroupWvalue":
        """
        class abc(object):
            pass

        args = abc()

        args.group_list = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/mcl/small_blast/80/all.80.group'
        args.bls_output = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/mcl/small_blast/all.bls'
        args.fasta_file = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/mcl/sj90'
        args.refgroup_seq = '/lustre/home/xuyuxing/Work/Kena/version3/mcl/cd_hit/mcl/small_blast/80/refgp.fa'
        """
        import networkx as nx

        group_dict = {}
        with open(args.group_list, 'r') as f:
            for each_line in f:
                each_line = re.sub('\n', '', each_line)
                group_id, seq_name = each_line.split("\t")
                if group_id not in group_dict:
                    group_dict[group_id] = []
                group_dict[group_id].append(seq_name)

        from pyfaidx import Fasta

        seqdict = Fasta(args.fasta_file)

        ref_seq_dict = {}
        for i in group_dict:
            ref_seq = sorted(group_dict[i], key=lambda x: len(
                seqdict[x]), reverse=True)[0]
            ref_seq_dict[i] = ref_seq

        with open(args.refgroup_seq, 'w') as f:
            for i in ref_seq_dict:
                f.write(">refgp_" + i + "\n" +
                        str(seqdict[ref_seq_dict[i]]) + "\n")

        import matplotlib.pyplot as plt
        import networkx as nx

        G = nx.Graph()

        G.add_nodes_from([1, 2, 3, 4])
        G.add_weighted_edges_from([(1, 2, 10), (3, 4, 10), (1, 3, 1)])
        pos = nx.spring_layout(G)

        plt.figure(figsize=(8, 8))
        nx.draw_networkx_nodes(G, pos,
                               node_size=80)
        nx.draw_networkx_labels(G, pos)
        plt.axis('off')
        plt.show()

        #########

        """
        class abc(object):
            pass
        
        args = abc()
        
        args.all_2_all_blast_results = '/Users/yuxingxu/Job/Work/FKN/all.tmp.bls'
        args.graph_pyb_file = '/Users/yuxingxu/Job/Work/FKN/graph.py'
        args.graph_pos_file ='/Users/yuxingxu/Job/Work/FKN/graph.pos.py' 
        """

        import matplotlib.pyplot as plt
        import networkx as nx
        import re
        import pickle

        G = nx.Graph()

        with open(args.all_2_all_blast_results, 'r') as r:
            num = 0
            for each_line in r:
                num = num + 1
                if re.match(r'^#', each_line):
                    continue
                each_line = re.sub('\n', '', each_line)
                info = each_line.split("\t")
                seq1, seq2, evalue = info[0], info[1], info[10]
                wvalue = evalue_to_wvalue(float(evalue))
                G.add_nodes_from([seq1, seq2])
                G.add_weighted_edges_from([(seq1, seq2, wvalue)])

        pos = nx.spring_layout(G)

        nx.write_gpickle(G, args.graph_pyb_file)
        with open(args.graph_pos_file, 'wb') as f:
            pickle.dump(pos, f)

        #####

        with open(args.graph_pos_file, 'rb') as f:
            pos = pickle.load(f)

        G = nx.read_gpickle(args.graph_pyb_file)

        plt.figure(figsize=(8, 8))
        nx.draw_networkx_nodes(G, pos,
                               node_size=10)
        # nx.draw_networkx_labels(G, pos)
        nx.draw_networkx_edges(G, pos, alpha=0.4)
        plt.axis('off')
        # plt.show()
        plt.savefig(args.output_file, dpi=1000)

    elif args_dict["subcommand_name"] == "GroupWvalue":
        pass

    elif args_dict["subcommand_name"] == "RenameNodeID":
        """
        class abc(object):
            pass

        args = abc()

        args.input_tree = '/lustre/home/xuyuxing/Work/Balanophora/speci_tree/genome/OrthoFinder/Results_Nov02/WorkingDirectory/Alignments_ids/SpeciesTreeAlignment.fa.raxml.bestTree'
        args.output_tree = '/lustre/home/xuyuxing/Work/Balanophora/speci_tree/genome/OrthoFinder/Results_Nov02/WorkingDirectory/Alignments_ids/SpeciesTreeAlignment.fa.raxml.bestTree.rename'
        args.ID_map_file = '/lustre/home/xuyuxing/Work/Balanophora/speci_tree/genome/OrthoFinder/Results_Nov02/WorkingDirectory/Alignments_ids/rename_map.txt'
        args.file_format = 'newick'
        """
        from api.xuyuxing.file_parser.fileIO import tsv_file_parse

        rename_map_dict = tsv_file_parse(args.ID_map_file, key_col=1)

        tree = Phylo.read(args.input_tree, args.file_format)

        for clade in tree.find_clades():
            if clade.name:
                if clade.name in rename_map_dict:
                    clade.name = rename_map_dict[clade.name][1]

        Phylo.write([tree], args.output_tree, args.file_format)

    elif args_dict["subcommand_name"] == "AddNodeID":
        """
        class abc(object):
            pass

        args = abc()

        args.input_tree = '/lustre/home/xuyuxing/Work/Other/liunaiyong/test/test.phb'
        args.output_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/xml/pr17.group.fasttree.node.xml'
        args.file_format = 'phyloxml'
        """

        tree = Phylo.read(args.input_tree, args.file_format)

        num = 0
        for clade in tree.find_clades():
            if clade.name is None:
                clade.name = "N%d" % num
                num = num + 1

        Phylo.write([tree], args.output_tree, args.file_format)

        # from ete3 import Tree

        # if args.file_format == 'newick':
        #     tree = Tree(args.input_tree, format=1)
        # elif args.file_format == 'phyloxml':
        #     tree = phyloxml_read_ete(args.input_tree)
        #
        # num = 0
        # for node in tree.traverse("postorder"):
        #     if not node.is_leaf():
        #         num = num + 1
        #         node.name = "N%d" % num
        #
        # if args.file_format == 'newick':
        #     tree.write(format=1, outfile=args.output_tree)
        # elif args.file_format == 'phyloxml':
        #     phyloxml_write_ete(tree, args.output_tree)

    elif args_dict["subcommand_name"] == "ClearNodeID":
        """
        class abc(object):
            pass

        args = abc()

        args.input_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/pr17.group.fasttree.phb'
        args.output_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/pr17.group.fasttree.nodeID.phb'
        """

        tree = Phylo.read(args.input_tree, args.file_format)

        for clade in tree.find_clades():
            if not clade.is_terminal():
                clade.name = None

        Phylo.write([tree], args.output_tree, args.file_format)

        # from ete3 import Tree

        # if args.file_format == 'newick':
        #     tree = Tree(args.input_tree, format=1)
        # elif args.file_format == 'phyloxml':
        #     tree = phyloxml_read_ete(args.input_tree)
        #
        # for node in tree.traverse("postorder"):
        #     if not node.is_leaf():
        #         node.name = ""
        #
        # if args.file_format == 'newick':
        #     tree.write(format=1, outfile=args.output_tree)
        # elif args.file_format == 'phyloxml':
        #     phyloxml_write_ete(tree, args.output_tree)

    elif args_dict["subcommand_name"] == "ClearComment":
        """
        class abc(object):
            pass

        args = abc()

        args.input_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/pr17.group.fasttree.phb'
        args.output_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/pr17.group.fasttree.nodeID.phb'
        """

        tree = Phylo.read(args.input_tree, args.file_format)

        for clade in tree.find_clades():
            if clade.comment:
                clade.comment = None

        Phylo.write([tree], args.output_tree, args.file_format)

    elif args_dict["subcommand_name"] == "ClearConfidence":
        """
        class abc(object):
            pass

        args = abc()

        args.input_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/pr17.group.fasttree.phb'
        args.output_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/pr17.group.fasttree.nodeID.phb'
        """

        tree = Phylo.read(args.input_tree, args.file_format)

        for clade in tree.find_clades():
            if clade.confidence:
                clade.confidence = None

        Phylo.write([tree], args.output_tree, args.file_format)

        # from ete3 import Tree

        # if args.file_format == 'newick':
        #     tree = Tree(args.input_tree, format=1)
        # elif args.file_format == 'phyloxml':
        #     tree = phyloxml_read_ete(args.input_tree)
        #
        # for node in tree.traverse("postorder"):
        #     if not node.is_leaf():
        #         node.name = ""
        #
        # if args.file_format == 'newick':
        #     tree.write(format=1, outfile=args.output_tree)
        # elif args.file_format == 'phyloxml':
        #     phyloxml_write_ete(tree, args.output_tree)

    elif args_dict["subcommand_name"] == "RootTree":
        """
        class abc(object):
            pass

        args = abc()

        args.node_name = "N390"
        args.input_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/xml/pr17.group.fasttree.node.xml'
        args.output_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/xml/pr17.group.fasttree.node.root.xml'
        """
        from lib.common.evolution.tree_operate import lookup_by_names

        tree = Phylo.read(args.input_tree, args.file_format)

        node_dict = lookup_by_names(tree)

        if args.node_name is None:
            tree.root_at_midpoint()
        else:
            tree.root_with_outgroup(node_dict[args.node_name])

        Phylo.write([tree], args.output_tree, args.file_format)

    elif args_dict["subcommand_name"] == "Newick2PhyloXML":
        """
        class abc(object):
            pass

        args = abc()

        args.input_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/pr17.group.fasttree.phb'
        args.output_tree = '/lustre/home/xuyuxing/Work/Kena/version2/mcl/pr17/cluster/all/pr17.group.fasttree/pr17.group.fasttree.xml'
        """

        Phylo.convert(args.input_tree, 'newick', args.output_tree, 'phyloxml')

    elif args_dict["subcommand_name"] == "DrawTree":
        """
        class abc(object):
            pass

        args = abc()
        args.tree_file = "/Users/yuxingxu/Downloads/SpeciesTreeAlignment.fa.raxml.bestTree"
        args.input_format = 'newick'
        args.output_file = "/Users/yuxingxu/Downloads/SpeciesTreeAlignment.fa.raxml.pdf"
        args.output_format = 'pdf'
        args.figure_size = (-20.0,20.0,-20.0,20.0)
        """

        tree = Phylo.read(args.tree_file, args.input_format)
        tree = normal_tree_rela_coord(tree)
        tree = normal_tree_abs_coord(tree, args.figure_size)

        fig, ax = plt.subplots(figsize=(21, 21))

        ax.set_xlim(-21, 21)
        ax.set_ylim(-21, 21)

        # ax.plot([10],[50],'ro')

        draw_tree(ax, tree)
        plt.savefig(args.output_file)
        # plt.show()

    elif args_dict["subcommand_name"] == "MonophylyTest":
        """
        class abc(object):
            pass

        args = abc()
        args.Bootstrap_trees = "/lustre/home/xuyuxing/Database/Balanophora/Comp_genome/dongming/orthofinder_191218/output/topo_test/subfile6.output/OG0010028/RAxML_bootstrap.OG0010028"
        args.file_format = 'newick'
        args.gene_list_file = '/lustre/home/xuyuxing/Database/Balanophora/Comp_genome/dongming/orthofinder_191218/output/topo_test/subfile6.output/OG0010028/tmp'
        """

        from lib.common.evolution.tree_operate import monophyly
        from lib.common.fileIO import read_list_file

        if not args.output_file is None:
            sys.stdout = open(args.output_file, 'w')

        gene_list = read_list_file(args.gene_list_file)

        for jacc_thre in [0.7, 0.75, 0.8, 0.85, 0.9, 0.95, 1]:
            tree_gen = Phylo.parse(args.Bootstrap_trees, args.file_format)
            num = 0
            for tree in tree_gen:
                flag, gene_list_tmp = monophyly(
                    gene_list, tree, None, jacc_thre)
                if flag:
                    num = num + 1
                # print(flag, len(gene_list_tmp))

            print("%.2f\t%d" % (jacc_thre, num))

    elif args_dict["subcommand_name"] == "RootTreeBySpecies":
        from Bio import Phylo
        from lib.common.fileIO import tsv_file_dict_parse
        from src.xuyuxing.tools.phytools import RootGeneTree

        file_info = tsv_file_dict_parse(args.gene_to_species_map, fieldnames=[
                                        'g_id', 's_id'], key_col='g_id')
        gene_to_species_map_dict = {}
        for i in file_info:
            gene_to_species_map_dict[i] = file_info[i]['s_id']

        gene_tree = Phylo.read(args.gene_tree, 'newick')
        species_tree = Phylo.read(args.species_tree, 'newick')

        gene_tree_rooted = RootGeneTree(
            gene_tree, species_tree, gene_to_species_map_dict)

        Phylo.write(gene_tree_rooted, args.output_tree, 'newick')

    elif args_dict["subcommand_name"] == "EasyABSREL":
        from src.xuyuxing.tools.hyphy_tools import easy_ABSREL
        import os

        args.rooted_gene_tree = os.path.realpath(args.rooted_gene_tree)
        args.cds_aln_fasta = os.path.realpath(args.cds_aln_fasta)
        args.output_prefix = os.path.realpath(args.output_prefix)

        easy_ABSREL(args.rooted_gene_tree, args.cds_aln_fasta, args.output_prefix,
                    tree_format=args.tree_format, threads=args.threads)

    elif args_dict["subcommand_name"] == "EasyRELAX":
        from src.xuyuxing.tools.hyphy_tools import easy_RELAX
        import os

        args.rooted_gene_tree = os.path.realpath(args.rooted_gene_tree)
        args.cds_aln_fasta = os.path.realpath(args.cds_aln_fasta)
        args.output_dir = os.path.realpath(args.output_dir)

        test_node_list = [i for i in args.test_node.split(",") if i != '']
        # print(test_node_list)

        easy_RELAX(args.rooted_gene_tree, args.cds_aln_fasta, args.output_dir,
                   test_node=test_node_list, tree_format=args.tree_format, threads=args.threads)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Retrieves the taxonomy file listing all organisms and the gene ID to KO mapping
files via the KEGG REST API for all organisms specified in the taxonomy file.
Modules will also be downloaded.
Files are saved as text files within a subdirectory. Existing files will be
overwritten.
Usage: buildKeggDB.py
"""
from api.xuyuxing.resource.kegg.kegg_config import *
import urllib.request
import os
import errno
import sys

def buildModuleList():
    # Prepare download directory. Create directory if it does not exist.
    try:
        os.makedirs(KEGGDIRNAME)
    except OSError as e:
        if e.errno != errno.EEXIST or not os.path.isdir(KEGGDIRNAME):
            raise

    # Download modules files from KEGG and save it.
    urllib.request.urlretrieve('http://rest.kegg.jp/list/module',
                       KEGGDIRNAME + '/ModuleList.txt')

    # Prepare file with modules and names and save it.
    try:
        os.remove(KEGGDIRNAME + '/ModulesNames.txt')
    except OSError:
        pass

    with open(KEGGDIRNAME + '/ModuleList.txt', 'r') as fin, open(
            KEGGDIRNAME + '/ModulesNames.txt', 'a') as fout:
        for line in fin.readlines():
            module = line.split('\t')[0].split(':')[1]
            name = line.split('\t')[1]
            fout.write(module + ': ' + name)

    # Prepare file with modules and save it.
    try:
        os.remove(KEGGDIRNAME + '/Modules.txt')
    except OSError:
        pass

    with open(KEGGDIRNAME + '/ModuleList.txt', 'r') as fin, open(KEGGDIRNAME +
                                                                 '/Modules.txt',
                                                                 'a') as fout:
        for line in fin.readlines():
            module = line.split('\t')[0].split(':')[1]
            fout.write(module + '\n')

def downloadTaxonomy():
    """
    Get taxonomy file with all organism codes from KEGG.
    """
    urllib.request.urlretrieve(TAXURL, KEGGDIRNAME + '/taxonomy.txt')


def buildOrgCodes(groups):
    """
    Save all organism codes in a list.
    Input: List of strings defning the groups for which taxonomy information
    will be downloaded (Prokaryotes, Eukaryotes)
    Output: List of organism codes
    """
    ORG_CODES = []
    TAXONOMY = []
    with open(KEGGDIRNAME + '/taxonomy.txt', 'r') as f:
        TAXONOMY = f.readlines()
    for line in TAXONOMY:
        assert len(line) > 0
        # get only those organisms from groups that are specified (Prokaryotes,
        # and/or Eukaryotes)
        if line.split('\t')[3].split(';')[0] in groups:
            # append the organism code to the list of all organisms of interest
            ORG_CODES.append(line.split('\t')[1])
    return ORG_CODES


def downloadGeneKO(organism):
    """
    Function to download the gene ID to KO mapping file for an organism via
    KEGG REST API.
    Input: string (organism code)
    Output: saved text file
    """
    urllib.request.urlretrieve(KOURL + organism, KEGGDIRNAME + '/' + organism + '.txt')


def downloadGeneEC(organism):
    """
    Function to download the gene ID to EC mapping file for an organism via
    KEGG REST API.
    Input: string (organism code)
    Output: saved text file
    """
    urllib.request.urlretrieve(ECURL + organism, KEGGDIRNAME + '/EC/'
                       + organism + '.txt')

def downloadGenePW(organism):
    """
    Function to download the gene ID to pathway mapping file for an organism via
    KEGG REST API.
    Input: string (organism code)
    Output: saved text file
    """
    urllib.request.urlretrieve(PWURL + organism, KEGGDIRNAME + '/EC/'
                       + organism + '.txt')

def downloadPW(organism):
    """
    Function to download the gene ID to pathway mapping file for an organism via
    KEGG REST API.
    Input: string (organism code)
    Output: saved text file
    """
    urllib.request.urlretrieve(PWURL + organism, KEGGDIRNAME + '/EC/'
                       + organism + '.txt')


# Main
if __name__ == '__main__':

    # ##### CUSTOMIZE #####
    # Directory where all downloaded KEGG data will be stored
    KEGGDIRNAME = '/lustre/home/xuyuxing/Database/kegg'

    # Directory where all downloaded KEGG EC data will be stored
    ECDIRNAME = KEGGDIRNAME + '/EC'

    # Directory where files with gene ID to KO mappings for each sample are stored
    MAPPINGSDIR = KEGGDIRNAME+'/mappings'

    # Directory where files with gene ID to EC mappings for each sample are stored
    ECMAPPINGSDIR = MAPPINGSDIR+'/EC'

    # Directory where module information for each sample is stored
    RESULTSDIR = KEGGDIRNAME+'/results'

    # Directory where EC information for each sample is stored
    ECRESULTSDIR = RESULTSDIR+'/EC'

    # specify Prokaryotes and/or Eukaryotes
    ORGANISMGROUP = ['Eukaryotes']
    # #####

    

    mkdir(KEGGDIRNAME, True)
    # Download taxonomy file
    downloadTaxonomy()
    # Download mapping files for all organisms in the list
    for organism in buildOrgCodes(ORGANISMGROUP):
        downloadGeneKO(organism)

    buildModuleList()


import os

# Genome_work_tools
project_dir = os.path.dirname(os.path.abspath(__file__))

# most used software path

fasttree_path = "FastTree"
raxml_path = "raxmlHPC-PTHREADS-SSE3"
clustalo_path = "clustalo"
clustalw_path = "clustalw2"
trimal_path = "trimal"
treebest_path = "treebest"
ascp_path = "/lustre/home/xuyuxing/.aspera/connect/bin/ascp"
wget_path = "wget"
mcl_path = ""
RNAfold_path = "RNAfold"
blast_path = ""
gce_dir_path = "/lustre/home/xuyuxing/Program/GCE/gce-1.0.0/"
samtools_path = "samtools"
fasta_path = "fasta36"
ssearch_path = "ssearch36"
genewise_path = "/lustre/home/xuyuxing/Program/wise/wise2.4.1/src/bin/genewise"
treeshrink_path = "run_treeshrink.py"
tmp_work_dir = "/tmp"
genblasta_path = "genblasta"
cd_hit_dir_path = "/lustre/home/xuyuxing/Program/cd-hit/cd-hit-v4.8.1-2019-0228"
pfam_db_file = '/lustre/home/xuyuxing/Database/Interpro/interproscan-5.39-77.0/data/pfam/32.0/pfam_a.hmm'
esearch_path = "esearch"
efetch_path = "efetch"
badirate_path = "/lustre/home/xuyuxing/Program/badirate/badirate-master/BadiRate.pl"
cutadapt_path = "cutadapt"
trf_path = 'trf'
diamond_path = 'diamond'

# wu_blast
wu_blast_path = '/lustre/home/xuyuxing/Program/wublast/'
genblasta_wu_script = '/lustre/home/xuyuxing/Program/genblast/genblasta_wu/genblasta_wu.sh'

# sqlite
sqlite_temp_store_directory = '/lustre/home/xuyuxing/.xyx/sqlite_tmp_folder'

# gene_painter
gene_painter_path = 'ruby /lustre/home/xuyuxing/Program/genepainter/gene_painter/gene_painter.rb'

# dlcpar_search
dlcpar_search_path = "dlcpar_search"

# ncbi taxon database
taxon_nr_db="/lustre/home/xuyuxing/Database/NCBI/nr/2020/nr.taxon.dmnd"
taxon_dump="/lustre/home/xuyuxing/Database/NCBI/nr/2020/taxdmp"

# Easy PlantTribes
hmmsearch_path="hmmsearch"
hmmbuild_path="hmmbuild"
hmmpress_path="hmmpress"
cap3_path="cap3"
transdecoder_path="TransDecoder.LongOrfs"
gt_path="gt"
plant_tribes_scaffold_dir="/lustre/home/xuyuxing/Program/PlantTribes/PlantTribes_scaffolds"
plant_tribes_default_scaffold="26Gv2.0"
assembly_post_processor='AssemblyPostProcessor'
assembly_post_processor_xyx='AssemblyPostProcessor.xyx'
gene_family_classifier='GeneFamilyClassifier'


"""
quick plot for mining data
    Yuxing Xu
    2020.04.19
"""
import matplotlib

matplotlib.use('Agg')
import matplotlib.pyplot as plt

__author__ = 'Yuxing Xu'

if __name__ == '__main__':
    import argparse

    ###### argument parse
    parser = argparse.ArgumentParser(
        prog='QuickPlot',
    )

    subparsers = parser.add_subparsers(title='subcommands', dest="subcommand_name")

    # argparse for Histogram
    parser_a = subparsers.add_parser('Histogram',
                                     help='get Histogram plot for a number list', description='')
    parser_a.add_argument("input_file", help="a file have a list of number (each line have one number)", type=str)
    parser_a.add_argument("-o", "--output_file", help="Output file name (default as hist_plot.pdf)",
                          default='hist_plot.pdf', type=str)

    # argparse for VennPlot
    parser_a = subparsers.add_parser('VennPlot',
                                     help='get venn plot for list of file', description='')
    parser_a.add_argument("input_file", nargs='*', help="a file have a list of object (each line have one object)",
                          type=str)
    parser_a.add_argument("-o", "--output_prefix", help="Output file name (default as venn)", default='venn', type=str)

    args = parser.parse_args()
    args_dict = vars(args)

    # --------------------------------------------
    #### command detail

    if args_dict["subcommand_name"] == "Histogram":
        from api.xuyuxing.plot.histogram import histogram_style1
        from lib.common.fileIO import read_list_file

        """
        class abc():
            pass
        
        args=abc()
        
        args.input_file = "/lustre/home/xuyuxing/Work/Other/qi/full_trans/X101SC19111988-Z01_PacbioRawdata_20191231/TZC_MIX/All/list1"
        """

        number_list = read_list_file(args.input_file)
        number_list = [float(i) for i in number_list]
        histogram_style1(number_list, args.output_file)

    elif args_dict["subcommand_name"] == "VennPlot":

        import api.xuyuxing.plot.VennDiagram as venn
        from lib.common.fileIO import read_list_file
        from collections import OrderedDict
        from lib.common.util import printer_list

        """
        class abc():
            pass

        args=abc()

        args.input_file = ["/lustre/home/xuyuxing/Database/Phelipanche/annotation/maker_p/maker_1M/busco/eudicots_miss_test/est.miss","/lustre/home/xuyuxing/Database/Phelipanche/annotation/maker_p/maker_1M/busco/eudicots_miss_test/aa.miss"]
        args.output_prefix = "/lustre/home/xuyuxing/Database/Phelipanche/annotation/maker_p/maker_1M/busco/eudicots_miss_test/venn"
        """

        venn_len = len(args.input_file)
        if venn_len < 2:
            raise ValueError("At least two list")
        else:
            data_dict = OrderedDict()
            tag_list = []
            for i in args.input_file:
                tag = i.split("/")[-1]
                tag_list.append(tag)
                list_tmp = read_list_file(i)
                data_dict[tag] = list_tmp

            labels, set_collections = venn.get_labels([data_dict[i] for i in tag_list], fill=['number'],
                                                      return_set_collections=True)

            text_report = args.output_prefix + ".txt"
            with open(text_report, 'w') as f:
                for code_tag in sorted(list(set_collections.keys()), key=lambda x: int(x), reverse=True):
                    decode_tag_list = []
                    for i in range(len(code_tag)):
                        if int(code_tag[i]):
                            decode_tag_list.append(tag_list[i])
                    # print(decode_tag_list)

                    f.write(printer_list(decode_tag_list, head="> ") + "\n")
                    f.write(printer_list(list(set_collections[code_tag]), wrap_num=20) + "\n")


            if venn_len == 2:
                venn_function = venn.venn2
            elif venn_len == 3:
                venn_function = venn.venn3
            elif venn_len == 4:
                venn_function = venn.venn4
            elif venn_len == 5:
                venn_function = venn.venn5
            elif venn_len == 6:
                venn_function = venn.venn6
            else:
                venn_function = None
                print("too many group, just do txt analysis")

            if venn_function:
                fig, ax = venn_function(labels, names=tag_list)
                fig.savefig(args.output_prefix + ".pdf", dpi=1000)
